@@ -49,6 +49,7 @@ static const GEnumValue _pgm_drawable_property_values[] = {
   { PGM_DRAWABLE_VISIBILITY, "PGM_DRAWABLE_VISIBILITY", "drawable-visibility" },
   { PGM_DRAWABLE_SIZE, "PGM_DRAWABLE_SIZE", "drawable-size" },
   { PGM_DRAWABLE_POSITION, "PGM_DRAWABLE_POSITION", "drawable-position" },
+  { PGM_DRAWABLE_TRANSFORMATION_MATRIX, "PGM_DRAWABLE_TRANSFORMATION_MATRIX", "drawable-transformation-matrix" },
   { PGM_DRAWABLE_BG_COLOR, "PGM_DRAWABLE_BG_COLOR", "drawable-bg-color" },
   { PGM_DRAWABLE_FG_COLOR, "PGM_DRAWABLE_FG_COLOR", "drawable-fg-color" },
   { PGM_DRAWABLE_OPACITY, "PGM_DRAWABLE_OPACITY", "drawable-opacity" },
@@ -56,8 +57,10 @@ static const GEnumValue _pgm_drawable_property_values[] = {
   { PGM_IMAGE_DATA_BUFFER, "PGM_IMAGE_DATA_BUFFER", "image-data-buffer" },
   { PGM_IMAGE_DATA_GST_BUFFER, "PGM_IMAGE_DATA_GST_BUFFER", "image-data-gst-buffer" },
   { PGM_IMAGE_DATA_PIXBUF, "PGM_IMAGE_DATA_PIXBUF", "image-data-pixbuf" },
+  { PGM_IMAGE_DATA_SYSTEM_BUFFER, "PGM_IMAGE_DATA_SYSTEM_BUFFER", "image-data-system-buffer" },
   { PGM_IMAGE_DATA_IMAGE, "PGM_IMAGE_DATA_IMAGE", "image-data-image" },
   { PGM_IMAGE_DATA_EMPTY, "PGM_IMAGE_DATA_EMPTY", "image-data-empty" },
+  { PGM_IMAGE_SYSTEM_BUFFER_CONTENT, "PGM_IMAGE_SYSTEM_BUFFER_CONTENT", "image-system-buffer-content" },
   { PGM_IMAGE_MAPPING_MATRIX, "PGM_IMAGE_MAPPING_MATRIX", "image-mapping-matrix" },
   { PGM_IMAGE_ALIGNMENT, "PGM_IMAGE_ALIGNMENT", "image-alignment" },
   { PGM_IMAGE_LAYOUT, "PGM_IMAGE_LAYOUT", "image-layout" },
@@ -116,6 +119,7 @@ pgm_drawable_flags_get_type (void)
 
 static const GFlagsValue _pgm_drawable_picking_signal_values[] = {
   { PGM_DRAWABLE_PICKING_PRESSED, "PGM_DRAWABLE_PICKING_PRESSED", "pressed" },
+  { PGM_DRAWABLE_PICKING_PRESSURED, "PGM_DRAWABLE_PICKING_PRESSURED", "pressured" },
   { PGM_DRAWABLE_PICKING_RELEASED, "PGM_DRAWABLE_PICKING_RELEASED", "released" },
   { PGM_DRAWABLE_PICKING_CLICKED, "PGM_DRAWABLE_PICKING_CLICKED", "clicked" },
   { PGM_DRAWABLE_PICKING_DOUBLE_CLICKED, "PGM_DRAWABLE_PICKING_DOUBLE_CLICKED", "double-clicked" },
@@ -123,8 +127,8 @@ static const GFlagsValue _pgm_drawable_picking_signal_values[] = {
   { PGM_DRAWABLE_PICKING_DRAG_MOTION, "PGM_DRAWABLE_PICKING_DRAG_MOTION", "drag-motion" },
   { PGM_DRAWABLE_PICKING_DRAG_END, "PGM_DRAWABLE_PICKING_DRAG_END", "drag-end" },
   { PGM_DRAWABLE_PICKING_ENTERED, "PGM_DRAWABLE_PICKING_ENTERED", "entered" },
-  { PGM_DRAWABLE_PICKING_LEFT, "PGM_DRAWABLE_PICKING_LEFT", "left" },
   { PGM_DRAWABLE_PICKING_SCROLLED, "PGM_DRAWABLE_PICKING_SCROLLED", "scrolled" },
+  { PGM_DRAWABLE_PICKING_MOTION, "PGM_DRAWABLE_PICKING_MOTION", "motion" },
   { PGM_DRAWABLE_PICKING_MASK, "PGM_DRAWABLE_PICKING_MASK", "mask" },
   { 0, NULL, NULL }
 };
@@ -149,6 +153,7 @@ static const GEnumValue _pgm_event_type_values[] = {
   { PGM_BUTTON_PRESS, "PGM_BUTTON_PRESS", "button-press" },
   { PGM_DOUBLE_BUTTON_PRESS, "PGM_DOUBLE_BUTTON_PRESS", "double-button-press" },
   { PGM_TRIPLE_BUTTON_PRESS, "PGM_TRIPLE_BUTTON_PRESS", "triple-button-press" },
+  { PGM_BUTTON_PRESSURE, "PGM_BUTTON_PRESSURE", "button-pressure" },
   { PGM_BUTTON_RELEASE, "PGM_BUTTON_RELEASE", "button-release" },
   { PGM_KEY_PRESS, "PGM_KEY_PRESS", "key-press" },
   { PGM_KEY_RELEASE, "PGM_KEY_RELEASE", "key-release" },
@@ -323,6 +328,7 @@ static const GEnumValue _pgm_image_storage_type_values[] = {
   { PGM_IMAGE_GST_BUFFER, "PGM_IMAGE_GST_BUFFER", "gst-buffer" },
   { PGM_IMAGE_PIXBUF, "PGM_IMAGE_PIXBUF", "pixbuf" },
   { PGM_IMAGE_IMAGE, "PGM_IMAGE_IMAGE", "image" },
+  { PGM_IMAGE_SYSTEM_BUFFER, "PGM_IMAGE_SYSTEM_BUFFER", "system-buffer" },
   { 0, NULL, NULL }
 };
 
@@ -511,6 +517,8 @@ static const GFlagsValue _pgm_viewport_capacity_values[] = {
   { PGM_VIEWPORT_HARDWARE_ACCELERATION, "PGM_VIEWPORT_HARDWARE_ACCELERATION", "hardware-acceleration" },
   { PGM_VIEWPORT_APPLICATION_EMBEDDING, "PGM_VIEWPORT_APPLICATION_EMBEDDING", "application-embedding" },
   { PGM_VIEWPORT_OPACITY, "PGM_VIEWPORT_OPACITY", "opacity" },
+  { PGM_VIEWPORT_TOUCHPAD, "PGM_VIEWPORT_TOUCHPAD", "touchpad" },
+  { PGM_VIEWPORT_X11_SYSTEM_BUFFER, "PGM_VIEWPORT_X11_SYSTEM_BUFFER", "x11-system-buffer" },
   { 0, NULL, NULL }
 };
 
@@ -525,6 +533,43 @@ pgm_viewport_capacity_get_type (void)
   return type;
 }
 
+static const GEnumValue _pgm_viewport_rotation_values[] = {
+  { PGM_VIEWPORT_ROTATION_NONE, "PGM_VIEWPORT_ROTATION_NONE", "none" },
+  { PGM_VIEWPORT_ROTATION_90, "PGM_VIEWPORT_ROTATION_90", "90" },
+  { PGM_VIEWPORT_ROTATION_180, "PGM_VIEWPORT_ROTATION_180", "180" },
+  { PGM_VIEWPORT_ROTATION_270, "PGM_VIEWPORT_ROTATION_270", "270" },
+  { 0, NULL, NULL }
+};
+
+GType
+pgm_viewport_rotation_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    type = g_enum_register_static ("PgmViewportRotation", _pgm_viewport_rotation_values);
+
+  return type;
+}
+
+static const GEnumValue _pgm_viewport_reflection_values[] = {
+  { PGM_VIEWPORT_REFLECTION_NONE, "PGM_VIEWPORT_REFLECTION_NONE", "none" },
+  { PGM_VIEWPORT_REFLECTION_HORIZONTAL_FLIP, "PGM_VIEWPORT_REFLECTION_HORIZONTAL_FLIP", "horizontal-flip" },
+  { PGM_VIEWPORT_REFLECTION_VERTICAL_FLIP, "PGM_VIEWPORT_REFLECTION_VERTICAL_FLIP", "vertical-flip" },
+  { 0, NULL, NULL }
+};
+
+GType
+pgm_viewport_reflection_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    type = g_enum_register_static ("PgmViewportReflection", _pgm_viewport_reflection_values);
+
+  return type;
+}
+
 
 /* enumerations from "./pgmlinearalgebra.h" */
 #include "./pgmlinearalgebra.h"
@@ -548,5 +593,26 @@ pgm_mat4x4_predefined_get_type (void)
 }
 
 
+/* enumerations from "./pgmimagesink.h" */
+#include "./pgmimagesink.h"
+static const GFlagsValue _pgm_image_sink_event_mask_values[] = {
+  { PGM_IMAGE_SINK_MOTION, "PGM_IMAGE_SINK_MOTION", "motion" },
+  { PGM_IMAGE_SINK_PRESSED, "PGM_IMAGE_SINK_PRESSED", "pressed" },
+  { PGM_IMAGE_SINK_RELEASED, "PGM_IMAGE_SINK_RELEASED", "released" },
+  { 0, NULL, NULL }
+};
+
+GType
+pgm_image_sink_event_mask_get_type (void)
+{
+  static GType type = 0;
+
+  if (!type)
+    type = g_flags_register_static ("PgmImageSinkEventMask", _pgm_image_sink_event_mask_values);
+
+  return type;
+}
+
+
 /* Generated data ends here */
 
