@@ -84,10 +84,20 @@ GType pgm_viewport_cursor_get_type (void);
 #define PGM_TYPE_VIEWPORT_CAPACITY pgm_viewport_capacity_get_type()
 GType pgm_viewport_capacity_get_type (void);
 
+#define PGM_TYPE_VIEWPORT_ROTATION pgm_viewport_rotation_get_type()
+GType pgm_viewport_rotation_get_type (void);
+
+#define PGM_TYPE_VIEWPORT_REFLECTION pgm_viewport_reflection_get_type()
+GType pgm_viewport_reflection_get_type (void);
+
 /* --- ./pgmlinearalgebra.h --- */
 #define PGM_TYPE_MAT4X4_PREDEFINED pgm_mat4x4_predefined_get_type()
 GType pgm_mat4x4_predefined_get_type (void);
 
+/* --- ./pgmimagesink.h --- */
+#define PGM_TYPE_IMAGE_SINK_EVENT_MASK pgm_image_sink_event_mask_get_type()
+GType pgm_image_sink_event_mask_get_type (void);
+
 G_END_DECLS
 
 #endif
