@@ -34,7 +34,6 @@ typedef struct _PgmEventKey       PgmEventKey;
 typedef struct _PgmEventExpose    PgmEventExpose;
 typedef struct _PgmEventConfigure PgmEventConfigure;
 typedef struct _PgmEventDnd       PgmEventDnd;
-typedef struct _PgmEventDelete    PgmEventDelete;
 typedef union  _PgmEvent          PgmEvent;
 
 #include <glib-object.h>
@@ -54,6 +53,7 @@ G_BEGIN_DECLS
  * @PGM_TRIPLE_BUTTON_PRESS: a mouse button has been clicked 3 times in a short
  * period of time. Note that each click also generates a %PGM_BUTTON_PRESS
  * event.
+ * @PGM_BUTTON_PRESSURE: a mouse button pressure has changed.
  * @PGM_BUTTON_RELEASE: a mouse button has been released.
  * @PGM_KEY_PRESS: a key has been pressed.
  * @PGM_KEY_RELEASE: a key has been released.
@@ -75,16 +75,17 @@ typedef enum {
   PGM_BUTTON_PRESS        = 1,
   PGM_DOUBLE_BUTTON_PRESS = 2,
   PGM_TRIPLE_BUTTON_PRESS = 3,
-  PGM_BUTTON_RELEASE      = 4,
-  PGM_KEY_PRESS           = 5,
-  PGM_KEY_RELEASE         = 6,
-  PGM_EXPOSE              = 7,
-  PGM_CONFIGURE           = 8,
-  PGM_DRAG_MOTION         = 9,
-  PGM_DRAG_DROP           = 10,
-  PGM_DRAG_LEAVE          = 11,
-  PGM_SCROLL              = 12,
-  PGM_DELETE              = 13
+  PGM_BUTTON_PRESSURE     = 4,
+  PGM_BUTTON_RELEASE      = 5,
+  PGM_KEY_PRESS           = 6,
+  PGM_KEY_RELEASE         = 7,
+  PGM_EXPOSE              = 8,
+  PGM_CONFIGURE           = 9,
+  PGM_DRAG_MOTION         = 10,
+  PGM_DRAG_DROP           = 12,
+  PGM_DRAG_LEAVE          = 13,
+  PGM_SCROLL              = 14,
+  PGM_DELETE              = 15
 } PgmEventType;
 
 /**
@@ -153,10 +154,9 @@ struct _PgmEventAny {
  * @source: the source field that can be filled by applications to indicate the
  * source of the event (mouse, pen, remote control, etc).
  * @time: the time of the event in milliseconds.
- * @x: the x coordinate of the pointer relative to the window in canvas
- * coordinate.
- * @y: the y coordinate of the pointer relative to the window in canvas
- * coordinate.
+ * @x: the x coordinate of the pointer relative to the window.
+ * @y: the y coordinate of the pointer relative to the window.
+ * @pressure: the pressure force, set to 0 when not used with a touch screen.
  *
  * Generated when the pointer moves.
  */
@@ -165,6 +165,7 @@ struct _PgmEventMotion {
   guint8       source;
   guint32      time;
   gfloat       x, y;
+  guint32      pressure;
 };
 
 /**
@@ -175,8 +176,8 @@ struct _PgmEventMotion {
  * @time: the time of the event in milliseconds.
  * @x: the x coordinate of the pointer relative to the window.
  * @y: the y coordinate of the pointer relative to the window.
- * coordinate.
  * @button: the button which was pressed or released.
+ * @pressure: the pressure force, set to 0 when not used with a touch screen.
  *
  * Used for button press and button release events.
  */
@@ -186,6 +187,7 @@ struct _PgmEventButton {
   guint32       time;
   gfloat        x, y;
   PgmButtonType button;
+  guint32       pressure;
 };
 
 /**
@@ -194,10 +196,8 @@ struct _PgmEventButton {
  * @source: the source field that can be filled by applications to indicate the
  * source of the event (mouse, pen, remote control, etc).
  * @time: the time of the event in milliseconds.
- * @x: the x coordinate of the pointer relative to the window in canvas
- * coordinate.
- * @y: the y coordinate of the pointer relative to the window in canvas
- * coordinate.
+ * @x: the x coordinate of the pointer relative to the window.
+ * @y: the y coordinate of the pointer relative to the window.
  * @direction: the scroll wheel direction.
  *
  * Generated when the mouse wheel is turned.
@@ -285,21 +285,6 @@ struct _PgmEventDnd {
   gchar        **uri;
 };
 
-/**
- * PgmEventDelete:
- * @type: the type of the event.
- * @source: the source field that can be filled by applications to indicate the
- * source of the event (mouse, pen, remote control, etc).
- * @time: the time of the event in milliseconds.
- *
- * Generated when the user requested that the viewport be destroyed.
- */
-struct _PgmEventDelete {
-  PgmEventType type;
-  guint8       source;
-  guint32      time;
-};
-
 /**
  * PgmEvent:
  *
@@ -351,7 +336,6 @@ union _PgmEvent {
   PgmEventExpose    expose;
   PgmEventConfigure configure;
   PgmEventDnd       dnd;
-  PgmEventDelete    delete;
 };
 
 GType     pgm_event_get_type    (void);
