@@ -196,6 +196,11 @@ do_clear (PgmImage *image)
       gdk_pixbuf_unref ((GdkPixbuf *) (image->data.pixbuf.pixbuf));
       break;
 
+    case PGM_IMAGE_SYSTEM_BUFFER:
+      GST_DEBUG_OBJECT (image, "Clearing system buffer image");
+      image->data.system_buffer.system_buffer = NULL;
+      break;
+
     case PGM_IMAGE_IMAGE:
       GST_DEBUG_OBJECT (image, "Clearing slaved image");
       if (image->master)
@@ -422,6 +427,105 @@ async_loader_free (AsyncLoader *async)
   g_slice_free (AsyncLoader, async);
 }
 
+/* Gets the width and height in pixel of the original image buffer.
+ * Called with LOCK. */
+static void
+image_get_buffer_pixel_size (PgmImage *image,
+                             guint *width,
+                             guint *height)
+{
+  switch (image->storage_type)
+    {
+    case PGM_IMAGE_BUFFER:
+      *width = image->data.buffer.width;
+      *height = image->data.buffer.height;
+      break;
+
+    case PGM_IMAGE_GST_BUFFER:
+      *width = image->data.gst_buffer.width;
+      *height = image->data.gst_buffer.height;
+      break;
+
+    case PGM_IMAGE_PIXBUF:
+      *width = gdk_pixbuf_get_width (image->data.pixbuf.pixbuf);
+      *height = gdk_pixbuf_get_height (image->data.pixbuf.pixbuf);
+      break;
+
+    case PGM_IMAGE_IMAGE:
+      image_get_buffer_pixel_size (image->master, width, height);
+      break;
+
+    case PGM_IMAGE_SYSTEM_BUFFER:
+      *width = image->data.system_buffer.width;
+      *height = image->data.system_buffer.height;
+      break;
+
+    default:
+      *width = *height = 0;
+    }
+}
+
+/* Gets the position of the upper left corner of the image in drawable
+ * coordinates (in x_offset and y_offset), and the resolution of the image, in
+ * pixel per canvas unit.
+ * Called with LOCK. */
+static void
+image_get_offset_and_resolution (PgmImage *image,
+                                 gfloat *x_offset,
+                                 gfloat *y_offset,
+                                 gfloat *x_resolution,
+                                 gfloat *y_resolution)
+{
+  PgmDrawable *drawable = PGM_DRAWABLE (image);
+  guint image_width, image_height;
+  gboolean image_wider, fill_height;
+  gfloat image_aspect_ratio;
+
+  image_get_buffer_pixel_size (image, &image_width, &image_height);
+
+  if (image->layout == PGM_IMAGE_FILLED)
+    {
+      *x_offset = 0;
+      *y_offset = 0;
+      *x_resolution = image_width / drawable->width;
+      *y_resolution = image_height / drawable->height;
+      return;
+    }
+
+  image_aspect_ratio = ((gfloat) image->par_n) / image->par_d;
+  image_wider = image_aspect_ratio > drawable->width / drawable->height;
+  fill_height = (image->layout == PGM_IMAGE_SCALED && !image_wider)
+                || (image->layout == PGM_IMAGE_ZOOMED && image_wider);
+
+  /* resolution */
+  if (fill_height)
+    {
+      *x_resolution = image_width / (image_aspect_ratio * drawable->height);
+      *y_resolution = image_height / drawable->height;
+    }
+  else /* fill width */
+    {
+      *x_resolution = image_width / drawable->width;
+      *y_resolution = image_height * image_aspect_ratio / drawable->width;
+    }
+
+  /* x_offset */
+  if (!fill_height || image->align & PGM_IMAGE_LEFT)
+    *x_offset = 0;
+  else if (image->align & PGM_IMAGE_RIGHT)
+    *x_offset = drawable->width - image_width / *x_resolution;
+  else /* PGM_IMAGE_CENTER */
+    *x_offset = 0.5f * (drawable->width - image_width / *x_resolution);
+
+  /* y_offset */
+  if (fill_height || image->align & PGM_IMAGE_TOP)
+    *y_offset = 0;
+  else if (image->align & PGM_IMAGE_BOTTOM)
+    *y_offset = drawable->height - image_height / *y_resolution;
+  else /* PGM_IMAGE_CENTER */
+    *y_offset = 0.5f * (drawable->height - image_height / *y_resolution);
+}
+
 /* GObject stuff */
 
 G_DEFINE_TYPE (PgmImage, pgm_image, PGM_TYPE_DRAWABLE);
@@ -712,7 +816,7 @@ pgm_image_set_from_buffer (PgmImage *image,
 
   /* The buffer sent is not the first of this size, it's not needed to call
    * a clear, just free the buffer. */
-  if (G_LIKELY (image->storage_type == PGM_IMAGE_GST_BUFFER
+  if (G_LIKELY (image->storage_type == PGM_IMAGE_BUFFER
                 && image->data.buffer.width == width
                 && image->data.buffer.height == height
                 && image->data.buffer.format == format
@@ -736,10 +840,10 @@ pgm_image_set_from_buffer (PgmImage *image,
 
       /* Store buffer informations */
       image->storage_type = PGM_IMAGE_BUFFER;
-      image->data.gst_buffer.format = format;
-      image->data.gst_buffer.width = width;
-      image->data.gst_buffer.height = height;
-      image->data.gst_buffer.stride = stride;
+      image->data.buffer.format = format;
+      image->data.buffer.width = width;
+      image->data.buffer.height = height;
+      image->data.buffer.stride = stride;
       image->data.buffer.size = size;
 
       /* Store ratio */
@@ -840,6 +944,65 @@ pgm_image_set_from_gst_buffer (PgmImage *image,
   return PGM_ERROR_OK;
 }
 
+/**
+ * pgm_image_set_from_system_buffer:
+ * @image: a #PgmImage object.
+ * @format: the pixel format of the buffer.
+ * @width: the image width in pixels.
+ * @height: the image height in pixels.
+ * @system_buffer: a pointer to the system buffer.
+ *
+ * Loads an image in @image from an existing system buffer.
+ *
+ * A system buffer depends on the platform, for instance on UNIX with an X11
+ * server running, @system_buffer can be set to an X11 Pixmap.
+ *
+ * MT safe.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_set_from_system_buffer (PgmImage *image,
+                                  PgmImagePixelFormat format,
+                                  guint width,
+                                  guint height,
+                                  gconstpointer system_buffer)
+{
+  g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
+  g_return_val_if_fail (system_buffer != NULL, PGM_ERROR_X);
+
+  /* Let's set the storage data */
+  GST_OBJECT_LOCK (image);
+
+  /* Let's clear if needed */
+  if (image->storage_type != PGM_IMAGE_EMPTY)
+    {
+      do_clear (image);
+      GST_OBJECT_UNLOCK (image);
+      _pgm_drawable_emit_changed (PGM_DRAWABLE (image), PGM_IMAGE_DATA_EMPTY);
+      GST_OBJECT_LOCK (image);
+    }
+
+  /* Store buffer informations */
+  image->storage_type = PGM_IMAGE_SYSTEM_BUFFER;
+  image->data.system_buffer.format = format;
+  image->data.system_buffer.width = width;
+  image->data.system_buffer.height = height;
+  image->data.system_buffer.system_buffer = system_buffer;
+
+  /* Store ratio */
+  image->par_n = width;
+  image->par_d = height;
+  update_slaves_ratio (image);
+
+  GST_OBJECT_UNLOCK (image);
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (image),
+                              PGM_IMAGE_DATA_SYSTEM_BUFFER);
+
+  return PGM_ERROR_OK;
+}
+
 /**
  * pgm_image_set_from_image:
  * @image: a #PgmImage object.
@@ -979,6 +1142,51 @@ pgm_image_get_storage_type (PgmImage *image,
   return PGM_ERROR_OK;
 }
 
+/**
+ * pgm_image_system_buffer_lock:
+ * @image: a #PgmImage object.
+ *
+ * Lock the system buffer set as the image content, ensuring the image is not
+ * going to be drawn while the system buffer content changes. This function
+ * guaranties tear free system buffer updates.
+ *
+ * MT safe.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_system_buffer_lock (PgmImage *image)
+{
+  g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
+
+  /* FIXME: lock */
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_image_system_buffer_unlock:
+ * @image: a #PgmImage object.
+ *
+ * Unlock the system buffer set as the image content.
+ *
+ * MT safe.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_system_buffer_unlock (PgmImage *image)
+{
+  g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
+
+  /* FIXME: unlock */
+
+  _pgm_drawable_emit_changed (PGM_DRAWABLE (image),
+                              PGM_IMAGE_SYSTEM_BUFFER_CONTENT);
+
+  return PGM_ERROR_OK;
+}
+
 /**
  * pgm_image_set_mapping_matrix:
  * @image: a #PgmImage object.
@@ -1516,3 +1724,83 @@ pgm_image_get_border_outer_color (PgmImage *image,
 
   return PGM_ERROR_OK;
 }
+
+/**
+ * pgm_image_from_drawable:
+ * @image: a #PgmImage instance
+ * @x_image: where to return the x coordinate in the @image plane.
+ * @y_image: where to return the y coordinate in the @image plane.
+ * @x_drawable: the x coordinate in drawable space.
+ * @y_drawable: the y coordinate in drawable space.
+ *
+ * Converts a position in drawable space (canvas units, origin at top left of
+ * drawable) into @image space (image pixel units, origin at top left of image).
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_from_drawable (PgmImage *image,
+                         gint *x_image,
+                         gint *y_image,
+                         gfloat x_drawable,
+                         gfloat y_drawable)
+{
+  gfloat x_offset, y_offset;
+  gfloat x_resolution, y_resolution;
+
+  g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
+  g_return_val_if_fail (NULL != x_image, PGM_ERROR_X);
+  g_return_val_if_fail (NULL != y_image, PGM_ERROR_X);
+  g_return_val_if_fail (image->layout != PGM_IMAGE_CENTERED
+                        || image->layout != PGM_IMAGE_TILED, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (image);
+  image_get_offset_and_resolution (image, &x_offset, &y_offset,
+                                   &x_resolution, &y_resolution);
+  GST_OBJECT_UNLOCK (image);
+
+  *x_image = (gint) ((x_drawable - x_offset) * x_resolution);
+  *y_image = (gint) ((y_drawable - y_offset) * y_resolution);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_image_to_drawable:
+ * @image: a #PgmImage instance
+ * @x_drawable: where to return the x coordinate in the @drawable plane.
+ * @y_drawable: where to return the y coordinate in the @drawable plane.
+ * @x_image: the x coordinate in image space.
+ * @y_image: the y coordinate in image space.
+ *
+ * Converts a position in @image space (image pixel units, origin at top left of
+ * image) into drawable space (canvas units, origin at top left of drawable).
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_to_drawable (PgmImage *image,
+                       gfloat *x_drawable,
+                       gfloat *y_drawable,
+                       gint x_image,
+                       gint y_image)
+{
+  gfloat x_offset, y_offset;
+  gfloat x_resolution, y_resolution;
+
+  g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
+  g_return_val_if_fail (NULL != x_drawable, PGM_ERROR_X);
+  g_return_val_if_fail (NULL != y_drawable, PGM_ERROR_X);
+  g_return_val_if_fail (image->layout != PGM_IMAGE_CENTERED
+                        || image->layout != PGM_IMAGE_TILED, PGM_ERROR_X);
+
+  GST_OBJECT_LOCK (image);
+  image_get_offset_and_resolution (image, &x_offset, &y_offset,
+                                   &x_resolution, &y_resolution);
+  GST_OBJECT_UNLOCK (image);
+
+  *x_drawable = x_offset + x_image / x_resolution;
+  *y_drawable = y_offset + y_image / y_resolution;
+
+  return PGM_ERROR_OK;
+}
