@@ -51,6 +51,7 @@
 #include <string.h>
 #include "pgmimagesink.h"
 #include "pgmcanvas.h"
+#include "pgmenumtypes.h"
 
 static GstElementDetails pgm_image_sink_details =
     GST_ELEMENT_DETAILS ("Video Sink", "Sink/Video",
@@ -74,7 +75,8 @@ GST_DEBUG_CATEGORY_STATIC (pgm_image_sink_debug);
 
 enum {
   ARG_0,
-  ARG_IMAGE
+  ARG_IMAGE,
+  ARG_EVENTS
 };
 
 static GstVideoSinkClass *parent_class = NULL;
@@ -333,6 +335,150 @@ update_format_from_caps (PgmImageSink *sink,
   return ret;
 }
 
+static gboolean
+push_navigation_event (PgmImageSink *image_sink,
+                       const gchar *event_name,
+                       gfloat x,
+                       gfloat y,
+                       gfloat z,
+                       PgmButtonType button)
+{
+  gboolean ret = FALSE;
+  GstStructure *structure;
+  GstEvent *event;
+  GstPad *pad;
+  gfloat x_drawable, y_drawable;
+  gint x_image, y_image;
+  guint gst_button;
+
+  /* Convert from canvas space to image space */
+  GST_OBJECT_LOCK (image_sink);
+  pgm_drawable_from_canvas (PGM_DRAWABLE (image_sink->image),
+                            &x_drawable, &y_drawable, x, y, z);
+  pgm_image_from_drawable (image_sink->image, &x_image, &y_image,
+                           x_drawable, y_drawable);
+  GST_OBJECT_UNLOCK (image_sink);
+
+  /* Convert from Pigment button to GStreamer button */
+  switch (button)
+    {
+    case PGM_BUTTON_LEFT:
+      gst_button = 1;
+    case PGM_BUTTON_MIDDLE:
+      gst_button = 2;
+    case PGM_BUTTON_RIGHT:
+      gst_button = 3;
+    default:
+      gst_button = 0;
+    }
+
+  /* Build and push the navigation event upstream */
+  if (gst_button != 0)
+    structure = gst_structure_new ("application/x-gst-navigation",
+                                   "event",     G_TYPE_STRING, event_name,
+                                   "pointer_x", G_TYPE_DOUBLE, (gdouble) x_image,
+                                   "pointer_y", G_TYPE_DOUBLE, (gdouble) y_image,
+                                   "button",    G_TYPE_INT,    gst_button,
+                                   NULL);
+  else
+    structure = gst_structure_new ("application/x-gst-navigation",
+                                   "event",     G_TYPE_STRING, event_name,
+                                   "pointer_x", G_TYPE_DOUBLE, (gdouble) x_image,
+                                   "pointer_y", G_TYPE_DOUBLE, (gdouble) y_image,
+                                   NULL);
+  event = gst_event_new_navigation (structure);
+  pad = gst_element_get_pad (GST_ELEMENT (image_sink), "sink");
+  ret = gst_pad_push_event (pad, event);
+
+  return ret;
+}
+
+static gboolean
+drawable_motion_cb (PgmDrawable *drawable,
+                    gfloat x,
+                    gfloat y,
+                    gfloat z,
+                    guint time,
+                    gpointer user_data)
+{
+  PgmImageSink *image_sink = PGM_IMAGE_SINK (user_data);
+
+  return push_navigation_event (image_sink, "mouse-move", x, y, z, 0);
+}
+
+static gboolean
+drawable_pressed_cb (PgmDrawable  *drawable,
+                     gfloat x,
+                     gfloat y,
+                     gfloat z,
+                     PgmButtonType button,
+                     guint time,
+                     guint pressure,
+                     gpointer user_data)
+{
+  PgmImageSink *image_sink = PGM_IMAGE_SINK (user_data);
+
+  return push_navigation_event (image_sink, "mouse-button-press",
+                                x, y, z, button);
+}
+
+static gboolean
+drawable_released_cb (PgmDrawable  *drawable,
+                      gfloat x,
+                      gfloat y,
+                      gfloat z,
+                      PgmButtonType button,
+                      guint time,
+                      gpointer user_data)
+{
+  PgmImageSink *image_sink = PGM_IMAGE_SINK (user_data);
+
+  return push_navigation_event (image_sink, "mouse-button-release",
+                                x, y, z, button);
+}
+
+/* Called with LOCK */
+static void
+update_event_mask (PgmImageSink *sink)
+{
+  if (NULL == sink->image)
+    return;
+
+  /* Update connection to the motion events */
+  if ((sink->events & PGM_IMAGE_SINK_MOTION) && !sink->motion_handler)
+    sink->motion_handler = g_signal_connect (sink->image, "motion",
+                                             G_CALLBACK (drawable_motion_cb),
+                                             sink);
+  else if (!(sink->events & PGM_IMAGE_SINK_MOTION) && sink->motion_handler)
+    {
+      g_signal_handler_disconnect (sink->image, sink->motion_handler);
+      sink->motion_handler = 0;
+    }
+
+  /* Update connection to the pressed events */
+  if ((sink->events & PGM_IMAGE_SINK_PRESSED) && !sink->pressed_handler)
+    sink->pressed_handler = g_signal_connect (sink->image, "pressed",
+                                              G_CALLBACK (drawable_pressed_cb),
+                                              sink);
+  else if (!(sink->events & PGM_IMAGE_SINK_PRESSED) && sink->pressed_handler)
+    {
+      g_signal_handler_disconnect (sink->image, sink->pressed_handler);
+      sink->pressed_handler = 0;
+    }
+
+  /* Update connection to the released events */
+  if ((sink->events & PGM_IMAGE_SINK_RELEASED) && !sink->released_handler)
+    sink->released_handler = g_signal_connect (sink->image, "released",
+                                               G_CALLBACK (drawable_released_cb),
+                                               sink);
+  else if (!(sink->events & PGM_IMAGE_SINK_RELEASED)
+           && sink->released_handler)
+    {
+      g_signal_handler_disconnect (sink->image, sink->released_handler);
+      sink->released_handler = 0;
+    }
+}
+
 /* GstBaseSink methods */
 
 static void
@@ -370,9 +516,9 @@ pgm_image_sink_show (GstBaseSink *bsink,
       pgm_image_set_from_gst_buffer (sink->image, sink->format, sink->width,
                                      sink->height, sink->stride, buf);
       /* And specify the pixel-aspect-ratio */
-      pgm_image_set_aspect_ratio (sink->image, sink->width *
-                                  ((gfloat) sink->par_n / sink->par_d),
-                                  sink->height);
+      pgm_image_set_aspect_ratio (sink->image, (guint) ((gfloat) sink->width *
+                                  ((gfloat) sink->par_n / (gfloat) sink->par_d)),
+                                  (guint) sink->height);
     }
 
   GST_OBJECT_UNLOCK (sink);
@@ -514,15 +660,38 @@ pgm_image_sink_set_property (GObject *object,
 
         /* Unref last image */
         if (G_UNLIKELY (sink->image))
-          gst_object_unref (sink->image);
+          {
+            if (sink->motion_handler)
+              g_signal_handler_disconnect (sink->image, sink->motion_handler);
+            if (sink->pressed_handler)
+              g_signal_handler_disconnect (sink->image, sink->pressed_handler);
+            if (sink->released_handler)
+              g_signal_handler_disconnect (sink->image, sink->released_handler);
+            gst_object_unref (sink->image);
+          }
 
         /* And get the new image */
         sink->image = gst_object_ref (PGM_IMAGE (g_value_get_object (value)));
+        update_event_mask (sink);
 
         GST_OBJECT_UNLOCK (sink);
         break;
       }
 
+    case ARG_EVENTS:
+      {
+        PgmImageSinkEventMask events = g_value_get_flags (value);
+
+        GST_OBJECT_LOCK (sink);
+        if (events != sink->events)
+          {
+            sink->events = events;
+            update_event_mask (sink);
+          }
+        GST_OBJECT_UNLOCK (sink);
+        break;
+      }
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -550,6 +719,12 @@ pgm_image_sink_get_property (GObject *object,
       GST_OBJECT_UNLOCK (sink);
       break;
 
+    case ARG_EVENTS:
+      GST_OBJECT_LOCK (sink);
+      g_value_set_flags (value, sink->events);
+      GST_OBJECT_UNLOCK (sink);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -565,6 +740,21 @@ pgm_image_sink_dispose (GObject *object)
 
   if (sink->image)
     {
+      if (sink->motion_handler)
+        {
+          g_signal_handler_disconnect (sink->image, sink->motion_handler);
+          sink->motion_handler = 0;
+        }
+      if (sink->pressed_handler)
+        {
+          g_signal_handler_disconnect (sink->image, sink->pressed_handler);
+          sink->pressed_handler = 0;
+        }
+      if (sink->released_handler)
+        {
+          g_signal_handler_disconnect (sink->image, sink->released_handler);
+          sink->released_handler = 0;
+        }
       gst_object_unref (sink->image);
       sink->image = NULL;
     }
@@ -608,10 +798,27 @@ pgm_image_sink_class_init (PgmImageSinkClass *klass)
   gobject_class->set_property = GST_DEBUG_FUNCPTR (pgm_image_sink_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (pgm_image_sink_get_property);
 
+  /**
+   * PgmImageSink:image:
+   *
+   * A #PgmImage on which the sink output video frames.
+   */
   g_object_class_install_property
     (gobject_class, ARG_IMAGE, g_param_spec_object
-     ("image", "PgmImage", "A PgmImage on which the sink ouput video frames",
+     ("image", "PgmImage", "A PgmImage on which the sink output video frames",
       PGM_TYPE_IMAGE, G_PARAM_READWRITE));
+  /**
+   * PgmImageSink:events:
+   *
+   * A #PgmImageSinkEventMask specifying what input events should be sent
+   * upstream in the pipeline.
+   */
+  g_object_class_install_property
+    (gobject_class, ARG_EVENTS, g_param_spec_flags
+     ("events", "PgmImageSinkEventMask", "A mask specifying what input events "
+      "should be sent upstream in the pipeline",
+      PGM_TYPE_IMAGE_SINK_EVENT_MASK, 0,
+      G_PARAM_READWRITE));
 
   gstelement_class->change_state =
     GST_DEBUG_FUNCPTR (pgm_image_sink_change_state);
@@ -631,6 +838,10 @@ pgm_image_sink_init (PgmImageSink *sink)
 {
   sink->image = NULL;
   sink->our_caps = NULL;
+  sink->motion_handler = 0;
+  sink->pressed_handler = 0;
+  sink->released_handler = 0;
+  sink->events = 0;
 }
 
 GType
@@ -650,6 +861,7 @@ pgm_image_sink_get_type (void)
         sizeof (PgmImageSink),
         0,
         (GInstanceInitFunc) pgm_image_sink_init,
+        NULL
       };
 
       sink_type = g_type_register_static (GST_TYPE_VIDEO_SINK, "PgmImageSink",
@@ -694,12 +906,8 @@ pgm_image_sink_set_image (PgmImageSink *sink,
   g_return_val_if_fail (PGM_IS_IMAGE_SINK (sink), PGM_ERROR_X);
   g_return_val_if_fail (PGM_IS_IMAGE (image), PGM_ERROR_X);
 
-  //GST_OBJECT_LOCK (sink);
-
   g_object_set (G_OBJECT (sink), "image", image, NULL);
 
-  //GST_OBJECT_UNLOCK (sink);
-
   return PGM_ERROR_OK;
 }
 
@@ -723,11 +931,50 @@ pgm_image_sink_get_image (PgmImageSink *sink,
   g_return_val_if_fail (PGM_IS_IMAGE_SINK (sink), PGM_ERROR_X);
   g_return_val_if_fail (image != NULL, PGM_ERROR_X);
 
-  GST_OBJECT_LOCK (sink);
-
   *image = gst_object_ref (sink->image);
 
-  GST_OBJECT_UNLOCK (sink);
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_image_sink_set_events:
+ * @sink: a #PgmImageSink object.
+ * @events: a #PgmImageSinkEventMask.
+ *
+ * Sets the mouse events interacting with the currently set image that will be
+ * sent upstream in the pipeline.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_sink_set_events (PgmImageSink *sink,
+                           PgmImageSinkEventMask events)
+{
+  g_return_val_if_fail (PGM_IS_IMAGE_SINK (sink), PGM_ERROR_X);
+
+  g_object_set (sink, "events", events, NULL);
+
+  return PGM_ERROR_OK;
+}
+
+/**
+ * pgm_image_sink_get_events:
+ * @sink: a #PgmImageSink object.
+ * @events: a #PgmImageSinkEventMask pointer where the event mask will be
+ * stored.
+ *
+ * Retrieves in @events the mouse events sent upstream in the pipeline.
+ *
+ * Returns: a #PgmError indicating success/failure.
+ */
+PgmError
+pgm_image_sink_get_events (PgmImageSink *sink,
+                           PgmImageSinkEventMask *events)
+{
+  g_return_val_if_fail (PGM_IS_IMAGE_SINK (sink), PGM_ERROR_X);
+  g_return_val_if_fail (NULL != events, PGM_ERROR_X);
+
+  g_object_get (sink, "events", events, NULL);
 
   return PGM_ERROR_OK;
 }
