@@ -193,54 +193,7 @@ pgm_marshal_VOID__OBJECT_OBJECT_INT (GClosure     *closure,
             data2);
 }
 
-/* BOOLEAN:ENUM,UINT (./pgmmarshal.list:33) */
-extern void pgm_marshal_BOOLEAN__ENUM_UINT (GClosure     *closure,
-                                            GValue       *return_value,
-                                            guint         n_param_values,
-                                            const GValue *param_values,
-                                            gpointer      invocation_hint,
-                                            gpointer      marshal_data);
-void
-pgm_marshal_BOOLEAN__ENUM_UINT (GClosure     *closure,
-                                GValue       *return_value,
-                                guint         n_param_values,
-                                const GValue *param_values,
-                                gpointer      invocation_hint,
-                                gpointer      marshal_data)
-{
-  typedef gboolean (*GMarshalFunc_BOOLEAN__ENUM_UINT) (gpointer     data1,
-                                                       gint         arg_1,
-                                                       guint        arg_2,
-                                                       gpointer     data2);
-  register GMarshalFunc_BOOLEAN__ENUM_UINT callback;
-  register GCClosure *cc = (GCClosure*) closure;
-  register gpointer data1, data2;
-  gboolean v_return;
-
-  g_return_if_fail (return_value != NULL);
-  g_return_if_fail (n_param_values == 3);
-
-  if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-      data1 = closure->data;
-      data2 = g_value_peek_pointer (param_values + 0);
-    }
-  else
-    {
-      data1 = g_value_peek_pointer (param_values + 0);
-      data2 = closure->data;
-    }
-  callback = (GMarshalFunc_BOOLEAN__ENUM_UINT) (marshal_data ? marshal_data : cc->callback);
-
-  v_return = callback (data1,
-                       g_marshal_value_peek_enum (param_values + 1),
-                       g_marshal_value_peek_uint (param_values + 2),
-                       data2);
-
-  g_value_set_boolean (return_value, v_return);
-}
-
-/* BOOLEAN:FLOAT,FLOAT,FLOAT,ENUM,UINT (./pgmmarshal.list:34) */
+/* BOOLEAN:FLOAT,FLOAT,FLOAT,ENUM,UINT (./pgmmarshal.list:33) */
 extern void pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT (GClosure     *closure,
                                                               GValue       *return_value,
                                                               guint         n_param_values,
@@ -293,6 +246,61 @@ pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
+/* BOOLEAN:FLOAT,FLOAT,FLOAT,ENUM,UINT,UINT (./pgmmarshal.list:34) */
+extern void pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT_UINT (GClosure     *closure,
+                                                                   GValue       *return_value,
+                                                                   guint         n_param_values,
+                                                                   const GValue *param_values,
+                                                                   gpointer      invocation_hint,
+                                                                   gpointer      marshal_data);
+void
+pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT_UINT (GClosure     *closure,
+                                                       GValue       *return_value,
+                                                       guint         n_param_values,
+                                                       const GValue *param_values,
+                                                       gpointer      invocation_hint,
+                                                       gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT_UINT) (gpointer     data1,
+                                                                              gfloat       arg_1,
+                                                                              gfloat       arg_2,
+                                                                              gfloat       arg_3,
+                                                                              gint         arg_4,
+                                                                              guint        arg_5,
+                                                                              guint        arg_6,
+                                                                              gpointer     data2);
+  register GMarshalFunc_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT_UINT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 7);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__FLOAT_FLOAT_FLOAT_ENUM_UINT_UINT) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_float (param_values + 1),
+                       g_marshal_value_peek_float (param_values + 2),
+                       g_marshal_value_peek_float (param_values + 3),
+                       g_marshal_value_peek_enum (param_values + 4),
+                       g_marshal_value_peek_uint (param_values + 5),
+                       g_marshal_value_peek_uint (param_values + 6),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
 /* BOOLEAN:FLOAT,FLOAT,FLOAT,UINT (./pgmmarshal.list:35) */
 extern void pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_UINT (GClosure     *closure,
                                                          GValue       *return_value,
@@ -344,7 +352,54 @@ pgm_marshal_BOOLEAN__FLOAT_FLOAT_FLOAT_UINT (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* VOID:UINT,UINT,POINTER (./pgmmarshal.list:36) */
+/* VOID:FLOAT,FLOAT,FLOAT,UINT (./pgmmarshal.list:36) */
+extern void pgm_marshal_VOID__FLOAT_FLOAT_FLOAT_UINT (GClosure     *closure,
+                                                      GValue       *return_value,
+                                                      guint         n_param_values,
+                                                      const GValue *param_values,
+                                                      gpointer      invocation_hint,
+                                                      gpointer      marshal_data);
+void
+pgm_marshal_VOID__FLOAT_FLOAT_FLOAT_UINT (GClosure     *closure,
+                                          GValue       *return_value,
+                                          guint         n_param_values,
+                                          const GValue *param_values,
+                                          gpointer      invocation_hint,
+                                          gpointer      marshal_data)
+{
+  typedef void (*GMarshalFunc_VOID__FLOAT_FLOAT_FLOAT_UINT) (gpointer     data1,
+                                                             gfloat       arg_1,
+                                                             gfloat       arg_2,
+                                                             gfloat       arg_3,
+                                                             guint        arg_4,
+                                                             gpointer     data2);
+  register GMarshalFunc_VOID__FLOAT_FLOAT_FLOAT_UINT callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+
+  g_return_if_fail (n_param_values == 5);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_VOID__FLOAT_FLOAT_FLOAT_UINT) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_marshal_value_peek_float (param_values + 1),
+            g_marshal_value_peek_float (param_values + 2),
+            g_marshal_value_peek_float (param_values + 3),
+            g_marshal_value_peek_uint (param_values + 4),
+            data2);
+}
+
+/* VOID:UINT,UINT,POINTER (./pgmmarshal.list:37) */
 extern void pgm_marshal_VOID__UINT_UINT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
