@@ -2,7 +2,7 @@
  *
  * Pigment media rendering library
  *
- * Copyright © 2006, 2007 Fluendo Embedded S.L.
+ * Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,7 +26,7 @@
 /**
  * SECTION:pgmtext
  * @short_description: A text drawable.
- * @see_also: #PgmDrawable.
+ * @see_also: #PgmDrawable, #PgmImage.
  *
  * <refsect2>
  * <para>
@@ -51,7 +51,7 @@
 #include <string.h>
 #include "pgmtext.h"
 
-GST_DEBUG_CATEGORY (pgm_text_debug);
+GST_DEBUG_CATEGORY_STATIC (pgm_text_debug);
 #define GST_CAT_DEFAULT pgm_text_debug
 
 static PgmDrawableClass *parent_class = NULL;
@@ -100,7 +100,7 @@ pgm_text_init (PgmText* text)
   text->style = PGM_TEXT_STYLE_NORMAL;
   text->variant = PGM_TEXT_VARIANT_NORMAL;
   text->weight = PGM_TEXT_WEIGHT_NORMAL;
-  text->height = 0.1f;
+  text->height = 1.0f;
 
   /* Text adjustment */
   text->ellipsize = PGM_TEXT_ELLIPSIZE_NONE;
@@ -307,8 +307,11 @@ pgm_text_get_font_family (PgmText *text,
  * @text: a #PgmText object.
  * @font_height: the text height in canvas-coordinates.
  *
- * Sets the height of @text to @height in canvas coordinates. The default
- * height is 0.25f.
+ * Sets the height of @text to @font_height in proportion to the height of the
+ * drawable. The default height is 1.0f, which fills the drawable in height.
+ * @font_height will be the maximum height of a line of text (i.e. the height of
+ * something like "Čq", measured from the bottom of the 'q' to the top of the
+ * accent on the 'Č').
  *
  * MT safe.
  *
