@@ -26,7 +26,7 @@
 #ifndef __PGM_VIEWPORT_H__
 #define __PGM_VIEWPORT_H__
 
-/* pgmviewport.h and pgmviewportfactory.h include eachother */
+/* pgmviewport.h and pgmviewportfactory.h include each other */
 typedef struct _PgmViewport      PgmViewport;
 typedef struct _PgmViewportClass PgmViewportClass;
 
@@ -73,6 +73,10 @@ typedef enum {
  * application.
  * @PGM_VIEWPORT_OPACITY: Supports opacity so that compositing managers can
  * blend the viewport with other applications.
+ * @PGM_VIEWPORT_TOUCHPAD: Supports touchpad so that application can use the
+ * pressure events and parameters.
+ * @PGM_VIEWPORT_X11_SYSTEM_BUFFER: Supports X11 system buffer as image content.
+ * Actually, X11 system buffers are Pixmaps.
  *
  * Capacities supported by the viewport.
  */
@@ -80,8 +84,40 @@ typedef enum {
   PGM_VIEWPORT_HARDWARE_ACCELERATION = (1 << 0),
   PGM_VIEWPORT_APPLICATION_EMBEDDING = (1 << 1),
   PGM_VIEWPORT_OPACITY               = (1 << 2),
+  PGM_VIEWPORT_TOUCHPAD              = (1 << 3),
+  PGM_VIEWPORT_X11_SYSTEM_BUFFER     = (1 << 4)
 } PgmViewportCapacity;
 
+/**
+ * PgmViewportRotation:
+ * @PGM_VIEWPORT_ROTATION_NONE: Do not rotate to the canvas.
+ * @PGM_VIEWPORT_ROTATION_90: Rotate the canvas 90° counter-clockwise.
+ * @PGM_VIEWPORT_ROTATION_180: Rotate the canvas 180° counter-clockwise.
+ * @PGM_VIEWPORT_ROTATION_270: Rotate the canvas 270° counter-clockwise.
+ *
+ * The rotation types of the canvas projection.
+ */
+typedef enum {
+  PGM_VIEWPORT_ROTATION_NONE = 0,
+  PGM_VIEWPORT_ROTATION_90   = 1,
+  PGM_VIEWPORT_ROTATION_180  = 2,
+  PGM_VIEWPORT_ROTATION_270  = 3
+} PgmViewportRotation;
+
+/**
+ * PgmViewportReflection:
+ * @PGM_VIEWPORT_REFLECTION_NONE: Do not reflect to the canvas.
+ * @PGM_VIEWPORT_REFLECTION_HORIZONTAL_FLIP: Reflect the canvas horizontally.
+ * @PGM_VIEWPORT_REFLECTION_VERTICAL_FLIP: Reflect the canvas vertically.
+ *
+ * The reflection types of the canvas projection.
+ */
+typedef enum {
+  PGM_VIEWPORT_REFLECTION_NONE            = 0,
+  PGM_VIEWPORT_REFLECTION_HORIZONTAL_FLIP = 1,
+  PGM_VIEWPORT_REFLECTION_VERTICAL_FLIP   = 2
+} PgmViewportReflection;
+
 /* FIXME: Should not be exported in this header since it's only used
  *        internally. That could be fixed separating public and private
  *        GObject fields (see #183) */
@@ -115,6 +151,8 @@ struct __PgmIOWatch {
  * @projected_y: the attached canvas projected y position.
  * @projected_width: the attached canvas projected width.
  * @projected_height: the attached canvas projected height.
+ * @rotation: the rotation of the projected canvas.
+ * @reflection: the reflection of the projected canvas.
  * @pixel_aspect_ratio: the pixel-aspect-ratio.
  * @viewport_ratio: the viewport ratio.
  * @canvas_ratio: the attached canvas ratio with pixel-aspect-ratio correction
@@ -146,6 +184,9 @@ struct _PgmViewport {
   gint       projected_x, projected_y;
   gint       projected_width, projected_height;
 
+  PgmViewportRotation   rotation;
+  PgmViewportReflection reflection;
+
   gfloat pixel_aspect_ratio;
   gfloat viewport_ratio;
   gfloat canvas_ratio;
@@ -165,6 +206,10 @@ struct _PgmViewport {
   GMutex      *pixel_lock;
   GList       *pixel_list;
 
+  /* Canvas rotation/reflection */
+  gfloat projection_hflip, projection_vflip;
+  gfloat projection_rotation;
+
   /* Signal handler id */
   gulong canvas_size_handler;
 
@@ -176,6 +221,7 @@ struct _PgmViewport {
  * PgmViewportClass:
  * @parent_class: the parent class structure.
  * @button_press_event: the "button-press-event" signal.
+ * @button_pressure_event: the "button-pressure-event" signal.
  * @button_release_event: the "button-release-event" signal.
  * @key_press_event: the "key-press-event" signal.
  * @key_release_event: the "key-release-event" signal.
@@ -188,6 +234,7 @@ struct _PgmViewport {
  * @drag_drop_event: the "drag-drop-event" signal.
  * @drag_leave_event: the "drag-leave-event" signal.
  * @pixels_read: the "pixels-read" signal.
+ * @update_pass: the "update-pass" signal.
  * @set_title: the set_title virtual method.
  * @show: the show virtual method.
  * @hide: the hide virtual method.
@@ -222,46 +269,51 @@ struct _PgmViewportClass {
 
   /* signals */
 
-  void     (*button_press_event)   (PgmViewport *viewport,
+  void     (*button_press_event)    (PgmViewport *viewport,
                                      PgmEventButton *event);
 
-  void     (*button_release_event) (PgmViewport *viewport,
-                                    PgmEventButton *event);
+  void     (*button_pressure_event) (PgmViewport *viewport,
+                                     PgmEventButton *event);
 
-  void     (*key_press_event)      (PgmViewport *viewport,
-                                    PgmEventKey *event);
+  void     (*button_release_event)  (PgmViewport *viewport,
+                                     PgmEventButton *event);
 
-  void     (*key_release_event)    (PgmViewport *viewport,
-                                    PgmEventKey *event);
+  void     (*key_press_event)       (PgmViewport *viewport,
+                                     PgmEventKey *event);
 
-  void     (*configure_event)      (PgmViewport *viewport,
-                                    PgmEventConfigure *event);
+  void     (*key_release_event)     (PgmViewport *viewport,
+                                     PgmEventKey *event);
 
-  void     (*delete_event)         (PgmViewport *viewport,
-                                    PgmEventDelete *event);
+  void     (*configure_event)       (PgmViewport *viewport,
+                                     PgmEventConfigure *event);
 
-  void     (*scroll_event)         (PgmViewport *viewport,
-                                    PgmEventScroll *event);
+  void     (*delete_event)          (PgmViewport *viewport,
+                                     PgmEvent *event);
 
-  void     (*motion_notify_event)  (PgmViewport *viewport,
-                                    PgmEventMotion *event);
+  void     (*scroll_event)          (PgmViewport *viewport,
+                                     PgmEventScroll *event);
 
-  void     (*expose_event)         (PgmViewport *viewport,
-                                    PgmEventExpose *event);
+  void     (*motion_notify_event)   (PgmViewport *viewport,
+                                     PgmEventMotion *event);
 
-  gboolean (*drag_motion_event)    (PgmViewport *viewport,
-                                    PgmEventDnd *event);
+  void     (*expose_event)          (PgmViewport *viewport,
+                                     PgmEventExpose *event);
 
-  void     (*drag_drop_event)      (PgmViewport *viewport,
-                                    PgmEventDnd *event);
+  gboolean (*drag_motion_event)     (PgmViewport *viewport,
+                                     PgmEventDnd *event);
 
-  void     (*drag_leave_event)     (PgmViewport *viewport,
-                                    PgmEventDnd *event);
+  void     (*drag_drop_event)       (PgmViewport *viewport,
+                                     PgmEventDnd *event);
 
-  void     (*pixels_read)          (PgmViewport *viewport,
-                                    guint width,
-                                    guint height,
-                                    gpointer pixels);
+  void     (*drag_leave_event)      (PgmViewport *viewport,
+                                     PgmEventDnd *event);
+
+  void     (*pixels_read)           (PgmViewport *viewport,
+                                     guint width,
+                                     guint height,
+                                     gpointer pixels);
+
+  void     (*update_pass)           (PgmViewport *viewport);
 
   /* virtual methods for subclasses */
 
@@ -415,6 +467,16 @@ PgmError     pgm_viewport_get_canvas            (PgmViewport *viewport,
 PgmError     pgm_viewport_set_canvas            (PgmViewport *viewport,
                                                  PgmCanvas *canvas);
 
+PgmError     pgm_viewport_set_canvas_rotation   (PgmViewport *viewport,
+                                                 PgmViewportRotation rotation);
+PgmError     pgm_viewport_get_canvas_rotation   (PgmViewport *viewport,
+                                                 PgmViewportRotation *rotation);
+
+PgmError     pgm_viewport_set_canvas_reflection (PgmViewport *viewport,
+                                                 PgmViewportReflection reflection);
+PgmError     pgm_viewport_get_canvas_reflection (PgmViewport *viewport,
+                                                 PgmViewportReflection *reflection);
+
 PgmError     pgm_viewport_update_projection     (PgmViewport *viewport);
 
 PgmError     pgm_viewport_to_canvas             (PgmViewport *viewport,
@@ -457,6 +519,8 @@ PgmError     pgm_viewport_push_pixels           (PgmViewport *viewport,
                                                  guint height,
                                                  guint8 *pixels);
 
+PgmError     pgm_viewport_emit_update_pass      (PgmViewport *viewport);
+
 G_END_DECLS
 
 #endif /* __PGM_VIEWPORT_H__ */
