@@ -455,3 +455,79 @@ pgm_backend_has_alpha_component (PgmBackend *backend)
 
   return FALSE;
 }
+
+gboolean
+pgm_backend_has_system_buffer (PgmBackend *backend)
+{
+  PgmBackendClass *klass;
+
+  g_return_val_if_fail (PGM_IS_BACKEND (backend), FALSE);
+
+  klass = PGM_BACKEND_GET_CLASS (backend);
+
+  if (klass->has_system_buffer)
+    return klass->has_system_buffer (backend);
+
+  return FALSE;
+}
+
+gpointer
+pgm_backend_create_system_buffer_object (PgmBackend *backend,
+                                         gconstpointer system_buffer,
+                                         PgmImagePixelFormat format)
+{
+  PgmBackendClass *klass;
+  gpointer system_buffer_object = NULL;
+
+  g_return_val_if_fail (PGM_IS_BACKEND (backend), system_buffer_object);
+
+  klass = PGM_BACKEND_GET_CLASS (backend);
+
+  if (klass->create_system_buffer_object)
+    system_buffer_object =
+      klass->create_system_buffer_object (backend, system_buffer, format);
+
+  return system_buffer_object;
+}
+
+void
+pgm_backend_destroy_system_buffer_object (PgmBackend *backend,
+                                          gpointer system_buffer_object)
+{
+  PgmBackendClass *klass;
+
+  g_return_if_fail (PGM_IS_BACKEND (backend));
+
+  klass = PGM_BACKEND_GET_CLASS (backend);
+
+  if (klass->destroy_system_buffer_object)
+    klass->destroy_system_buffer_object (backend, system_buffer_object);
+}
+
+void
+pgm_backend_bind_system_buffer_object (PgmBackend *backend,
+                                       gconstpointer system_buffer_object)
+{
+  PgmBackendClass *klass;
+
+  g_return_if_fail (PGM_IS_BACKEND (backend));
+
+  klass = PGM_BACKEND_GET_CLASS (backend);
+
+  if (klass->bind_system_buffer_object)
+    klass->bind_system_buffer_object (backend, system_buffer_object);
+}
+
+void
+pgm_backend_release_system_buffer_object (PgmBackend *backend,
+                                          gconstpointer system_buffer_object)
+{
+  PgmBackendClass *klass;
+
+  g_return_if_fail (PGM_IS_BACKEND (backend));
+
+  klass = PGM_BACKEND_GET_CLASS (backend);
+
+  if (klass->release_system_buffer_object)
+    klass->release_system_buffer_object (backend, system_buffer_object);
+}
