@@ -152,6 +152,27 @@ struct _PgmBackendClass {
 
   /* Retrieve if the backend handles alpha clear color */
   gboolean (*has_alpha_component)     (PgmBackend *backend);
+
+  /* Retrieve if the backend supports system buffers */
+  gboolean (*has_system_buffer)            (PgmBackend *backend);
+
+  /* Create a system buffer object understandable by the backend from a given
+   * system buffer */
+  gpointer (*create_system_buffer_object)  (PgmBackend *backend,
+                                            gconstpointer system_buffer,
+                                            PgmImagePixelFormat format);
+
+  /* Destroy the system buffer object created with create_system_buffer */
+  void     (*destroy_system_buffer_object) (PgmBackend *backend,
+                                            gpointer system_buffer_object);
+
+  /* Bind the system buffer object created with create_system_buffer */
+  void     (*bind_system_buffer_object)    (PgmBackend *backend,
+                                            gconstpointer system_buffer_object);
+
+  /* Release the system buffer object bound with bind_system_buffer */
+  void     (*release_system_buffer_object) (PgmBackend *backend,
+                                            gconstpointer system_buffer_object);
 };
 
 GType       pgm_backend_get_type                (void) G_GNUC_CONST;
@@ -229,6 +250,21 @@ void        pgm_backend_get_embedding_id        (PgmBackend *backend,
 
 gboolean    pgm_backend_has_alpha_component     (PgmBackend *backend);
 
+gboolean    pgm_backend_has_system_buffer            (PgmBackend *backend);
+
+gpointer    pgm_backend_create_system_buffer_object  (PgmBackend *backend,
+                                                      gconstpointer system_buffer,
+                                                      PgmImagePixelFormat format);
+
+void        pgm_backend_destroy_system_buffer_object (PgmBackend *backend,
+                                                      gpointer system_buffer_object);
+
+void        pgm_backend_bind_system_buffer_object    (PgmBackend *backend,
+                                                      gconstpointer system_buffer_object);
+
+void        pgm_backend_release_system_buffer_object (PgmBackend *backend,
+                                                      gconstpointer system_buffer_object);
+
 G_END_DECLS
 
 #endif /* __PGM_BACKEND_H__ */
