@@ -56,10 +56,9 @@
 
 #ifdef WIN32
 #include <windows.h>
-#include <fcntl.h>    /* _O_BINARY */
-#include <io.h>       /* _pipe, _close */
-#define pipe(handle)  _pipe(handle, 4096, _O_BINARY)
-#define close(handle) _close(handle)
+#include <fcntl.h>   /* _O_BINARY */
+#include <io.h>      /* _pipe */
+#define pipe(handle) _pipe(handle, 4096, _O_BINARY)
 #endif /* WIN32 */
 
 #include <GL/gl.h>  /* gl* */
@@ -784,7 +783,7 @@ do_read_pixels (PgmContext *context,
   render (context);
   gl->read_buffer (PGM_GL_BACK);
   gl->read_pixels (rectangle->x, rectangle->y, rectangle->width,
-                   rectangle->height, GL_RGBA, GL_UNSIGNED_BYTE,
+                   rectangle->height, PGM_GL_RGBA, PGM_GL_UNSIGNED_BYTE,
                    rectangle->pixels);
   gl->clear (PGM_GL_COLOR_BUFFER_BIT);
   gl->read_buffer (PGM_GL_FRONT);
@@ -792,9 +791,9 @@ do_read_pixels (PgmContext *context,
   /* Pop our inverted projection matrix if needed */
   if (G_LIKELY (canvas))
     {
-      context->gl->matrix_mode (PGM_GL_PROJECTION);
-      context->gl->pop_matrix ();
-      context->gl->matrix_mode (PGM_GL_MODELVIEW);
+      gl->matrix_mode (PGM_GL_PROJECTION);
+      gl->pop_matrix ();
+      gl->matrix_mode (PGM_GL_MODELVIEW);
     }
 
   /* Push the read pixels back to the viewport */
@@ -806,6 +805,36 @@ do_read_pixels (PgmContext *context,
   rectangle = NULL;
 }
 
+/* Quit the rendering thread */
+static void
+do_quit (PgmContext *context,
+         gpointer data)
+{
+  GSource *source;
+
+  /* Remove sources. Do not use g_source_remove() since this only works for
+   * sources attached to the default GMainContext. See bug #261. */
+  if (context->immediate_tag)
+    {
+      source = g_main_context_find_source_by_id (context->render_context,
+                                                 context->immediate_tag);
+      if (source)
+        g_source_destroy (source);
+    }
+  if (context->update_tag)
+    {
+      source = g_main_context_find_source_by_id (context->render_context,
+                                                 context->update_tag);
+      if (source)
+        g_source_destroy (source);
+    }
+
+  /* Join the rendering thread */
+  g_main_loop_quit (context->render_loop);
+  g_main_loop_unref (context->render_loop);
+  g_main_context_unref (context->render_context);
+}
+
 /* Texture generation task handler */
 static void
 do_gen_texture (PgmContext *context,
@@ -1007,6 +1036,9 @@ update_cb (gpointer data)
 {
   PgmContext *context = PGM_CONTEXT (data);
 
+  /* User defined update callback */
+  pgm_viewport_emit_update_pass (PGM_VIEWPORT (context->glviewport));
+
   /* Flush update queue */
   pgm_gl_viewport_flush_update_queue (context->glviewport);
 
@@ -1039,7 +1071,7 @@ update_removal_timeout_cb (gpointer data)
 
   /* If the elapsed time since the previous update request rises above one
    * second, remove the auto-update source */
-  if ((current_time.tv_sec - context->update_timestamp.tv_sec) >= 1)
+  if ((current_time.tv_sec - context->update_timestamp.tv_sec) >= 2)
     {
       GSource *source;
 
@@ -1091,6 +1123,10 @@ create_io_channels (gint *fd,
       GST_ERROR ("cannot create the output channel");
       return;
     }
+
+  /* We don't close the fd ourselves because otherwise it deadlocks on windows */
+  g_io_channel_set_close_on_unref (*in, TRUE);
+  g_io_channel_set_close_on_unref (*out, TRUE);
 }
 
 /* Close a pipe and unref its i/o channels */
@@ -1103,14 +1139,12 @@ delete_io_channels (gint *fd,
     {
       g_io_channel_unref (*out);
       *out = NULL;
-      close (fd[0]);
     }
 
   if (*in)
     {
       g_io_channel_unref (*in);
       *in = NULL;
-      close (fd[1]);
     }
 }
 
@@ -1162,7 +1196,11 @@ render_loop (gpointer data)
   context->backend = PGM_BACKEND (pgm_wgl_backend_new (context));
 #endif
 
-  pgm_backend_create_window (context->backend);
+  if (!pgm_backend_create_window (context->backend))
+    {
+      g_print ("ERROR: couldn't create the window, exiting...\n");
+      exit (1); /* FIXME: Improve error handling!!! */
+    }
 
   context->gl = gl = &gl_proc_address;
 
@@ -1181,15 +1219,15 @@ render_loop (gpointer data)
   /* Check that OpenGL minimum version is 1.2 */
   if (!context->version_string)
     {
-      GST_ERROR ("couldn't retrieve OpenGL version");
-      exit (1); /* FIXME: So bad!!! */
+      g_print ("ERROR: couldn't retrieve OpenGL version, exiting...\n");
+      exit (1); /* FIXME: Improve error handling!!! */
     }
   context->version = get_float_opengl_version (context->version_string);
   if (context->version < 1.2f)
     {
-      GST_ERROR ("OpenGL version %.1f (1.2 minimum required)",
-                 context->version);
-      exit (1); /* FIXME: So bad!!! */
+      g_print ("ERROR: OpenGL version %.1f (1.2 minimum required), exiting...\n",
+               context->version);
+      exit (1); /* FIXME: Improve error handling!!! */
     }
 
   /* Retrieve and bind supported extensions */
@@ -1384,6 +1422,7 @@ init_context (PgmContext *context)
     GST_DEBUG_FUNCPTR (do_update_texture);
   context->task_func[PGM_CONTEXT_FREE_TEXTURE] =
     GST_DEBUG_FUNCPTR (do_free_texture);
+  context->task_func[PGM_CONTEXT_QUIT] = GST_DEBUG_FUNCPTR (do_quit);
 
   /* Wait for the rendering thread initialization completion */
   g_mutex_lock (context->init_mutex);
@@ -1398,27 +1437,7 @@ init_context (PgmContext *context)
 static void
 dispose_context (PgmContext *context)
 {
-  /* Iterate the last events */
-  while (g_main_context_pending (context->render_context))
-    g_main_context_iteration (context->render_context, FALSE);
-
-  /* Remove the immediate source */
-  if (context->immediate_tag)
-    g_source_remove (context->immediate_tag);
-
-  /* Remove the auto-update source */
-  if (context->update_tag)
-    {
-      GSource *source = g_main_context_find_source_by_id
-        (context->render_context, context->update_tag);
-      if (source)
-        g_source_destroy (source);
-    }
-
   /* Join the rendering thread */
-  g_main_loop_quit (context->render_loop);
-  g_main_loop_unref (context->render_loop);
-  g_main_context_unref (context->render_context);
   g_thread_join (context->render_thread);
 
   /* Dispose immediate source data */
@@ -1532,7 +1551,7 @@ pgm_context_update (PgmContext *context)
       g_source_unref (source);
 
       /* Add the update removal timeout to the rendering GMainContext */
-      source = g_timeout_source_new (500);
+      source = g_timeout_source_new (750);
       g_source_set_callback (source, update_removal_timeout_cb, context, NULL);
       g_source_attach (source, context->render_context);
       g_source_unref (source);
