@@ -2,7 +2,7 @@
  *
  * Pigment OpenGL plugin
  *
- * Copyright © 2006, 2007 Fluendo Embedded S.L.
+ * Copyright © 2006, 2007, 2008 Fluendo Embedded S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -49,11 +49,12 @@ G_BEGIN_DECLS
 #define PGM_GL_FEAT_VERTEX_BUFFER_OBJECT     (1L << 10)
 #define PGM_GL_FEAT_PIXEL_BUFFER_OBJECT      (1L << 11)
 #define PGM_GL_FEAT_PER_PLANE_YCBCR_PROGRAM  (1L << 12)
-#define PGM_GL_FEAT_BLEND_COLOR              (1L << 13)
-#define PGM_GL_FEAT_PACKED_PIXELS            (1L << 14)
-#define PGM_GL_FEAT_MULTI_DRAW_ARRAYS        (1L << 15)
-#define PGM_GL_FEAT_FRAMEBUFFER_OBJECT       (1L << 16)
-#define PGM_GL_FEAT_COPY_SUB_BUFFER          (1L << 17)
+#define PGM_GL_FEAT_BLEND_FUNC_SEPARATE      (1L << 13)
+#define PGM_GL_FEAT_BLEND_COLOR              (1L << 14)
+#define PGM_GL_FEAT_PACKED_PIXELS            (1L << 15)
+#define PGM_GL_FEAT_MULTI_DRAW_ARRAYS        (1L << 16)
+#define PGM_GL_FEAT_FRAMEBUFFER_OBJECT       (1L << 17)
+#define PGM_GL_FEAT_COPY_SUB_BUFFER          (1L << 18)
 
 /* OpenGL constant definitions */
 
@@ -90,6 +91,7 @@ G_BEGIN_DECLS
 #define PGM_GL_POLYGON        0x0009
 
 #define PGM_GL_VERTEX_ARRAY        0x8074
+#define PGM_GL_COLOR_ARRAY         0x8076
 #define PGM_GL_TEXTURE_COORD_ARRAY 0x8078
 
 #define PGM_GL_FILL      0x1B02
@@ -209,6 +211,7 @@ G_BEGIN_DECLS
 
 #define PGM_GL_MODELVIEW_MATRIX  0x0BA6
 #define PGM_GL_PROJECTION_MATRIX 0x0BA7
+#define PGM_GL_TEXTURE_MATRIX    0x0BA8
 
 #define PGM_GL_FRONT_AND_BACK 0x0408
 #define PGM_GL_FLAT           0x1D00
@@ -379,6 +382,8 @@ typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_disable_client_state)
   (PgmGlEnum cap);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_vertex_pointer)
   (PgmGlInt size, PgmGlEnum type, PgmGlSizei stride, const PgmGlVoid *ptr);
+typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_color_pointer)
+  (PgmGlInt size, PgmGlEnum type, PgmGlSizei stride, const PgmGlVoid *ptr);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_tex_coord_pointer)
   (PgmGlInt size, PgmGlEnum type, PgmGlSizei stride, const PgmGlVoid *ptr);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_draw_arrays)
@@ -403,6 +408,8 @@ typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_depth_func)
   (PgmGlEnum function);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_blend_func)
   (PgmGlEnum sfactor, PgmGlEnum dfactor);
+typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_blend_func_separate)
+  (PgmGlEnum srgb, PgmGlEnum drgb, PgmGlEnum salpha, PgmGlEnum dalpha);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_blend_color)
   (PgmGlClampf red, PgmGlClampf green, PgmGlClampf blue, PgmGlClampf alpha);
 typedef PgmGlVoid (PGM_GL_API_ATTRIBUTE * pgm_gl_line_width)
