@@ -688,6 +688,9 @@ pgm_gl_image_sync (PgmGlDrawable *gldrawable)
     case PGM_IMAGE_PIXBUF:
       pgm_gl_image_set_from_pixbuf (glimage);
       break;
+    case PGM_IMAGE_SYSTEM_BUFFER:
+      pgm_gl_image_set_from_system_buffer (glimage);
+      break;
     case PGM_IMAGE_IMAGE:
       pgm_gl_image_set_from_image (glimage);
       break;
@@ -1010,6 +1013,61 @@ pgm_gl_image_set_from_pixbuf (PgmGlImage *glimage)
   pgm_context_push_deferred_task (gldrawable->glviewport->context, task);
 }
 
+void
+pgm_gl_image_set_from_system_buffer (PgmGlImage *glimage)
+{
+  PgmGlDrawable *gldrawable = PGM_GL_DRAWABLE (glimage);
+  PgmImage *image = PGM_IMAGE (gldrawable->drawable);
+  PgmContextTask *task;
+
+  GST_DEBUG_OBJECT (glimage, "set_from_system_buffer");
+
+  GST_OBJECT_LOCK (image);
+
+  if (!(gldrawable->glviewport->context->feature_mask
+        & PGM_GL_FEAT_TEXTURE_NON_POWER_OF_TWO))
+    {
+      GST_DEBUG_OBJECT (image, "System buffer cannot be set, the OpenGL "
+                        "implementation does not support NPOT textures");
+      GST_OBJECT_UNLOCK (image);
+      return;
+    }
+
+  if (!(image->data.system_buffer.format == PGM_IMAGE_RGB
+        || image->data.system_buffer.format == PGM_IMAGE_RGBA))
+    {
+      GST_DEBUG_OBJECT (image, "System buffer cannot be set, the OpenGL plugin "
+                        "only supports RGB and RGBA color spaces");
+      GST_OBJECT_UNLOCK (image);
+      return;
+    }
+
+  if (G_UNLIKELY (!(image->storage_type == PGM_IMAGE_SYSTEM_BUFFER
+                    || image->storage_type == PGM_IMAGE_IMAGE)))
+    {
+      GST_OBJECT_UNLOCK (image);
+      return;
+    }
+
+  pgm_texture_set_system_buffer (glimage->texture,
+                                 image->data.system_buffer.system_buffer,
+                                 image->data.system_buffer.format,
+                                 image->data.system_buffer.width,
+                                 image->data.system_buffer.height);
+  pgm_texture_set_matrix (glimage->texture, image->mapping_matrix);
+
+  GST_OBJECT_UNLOCK (image);
+
+  glimage->empty = FALSE;
+
+  update_image_ratio (glimage);
+  update_layout (glimage);
+  update_slaves (glimage);
+
+  task = pgm_context_task_new (PGM_CONTEXT_GEN_TEXTURE, glimage->texture);
+  pgm_context_push_immediate_task (gldrawable->glviewport->context, task);
+}
+
 void
 pgm_gl_image_set_from_image (PgmGlImage *glimage)
 {
@@ -1116,3 +1174,9 @@ pgm_gl_image_set_border_outer_color (PgmGlImage *glimage)
 
   update_border_outer_color (glimage);
 }
+
+void
+pgm_gl_image_system_buffer_changed (PgmGlImage *glimage)
+{
+  GST_DEBUG_OBJECT (glimage, "system_buffer_changed");
+}
