@@ -91,7 +91,13 @@ get_projected_height (PgmGlDrawable *gldrawable,
   /* FIXME: That's just the projected height on z=0 */
   GST_OBJECT_LOCK (canvas);
   GST_OBJECT_LOCK (viewport);
-  height = (canvas_height * viewport->projected_height) / canvas->height;
+
+  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
+      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
+    height = (canvas_height * viewport->projected_height) / canvas->height;
+  else
+    height = (canvas_height * viewport->projected_height) / canvas->width;
+
   GST_OBJECT_UNLOCK (viewport);
   GST_OBJECT_UNLOCK (canvas);
 
@@ -101,7 +107,7 @@ get_projected_height (PgmGlDrawable *gldrawable,
 /* Gets the projected width of the drawable on the viewport */
 static gfloat
 get_projected_width (PgmGlDrawable *gldrawable,
-                           gfloat canvas_width)
+                     gfloat canvas_width)
 {
   PgmGlViewport *glviewport = gldrawable->glviewport;
   PgmViewport *viewport = PGM_VIEWPORT (glviewport);
@@ -111,7 +117,13 @@ get_projected_width (PgmGlDrawable *gldrawable,
   /* FIXME: That's just the projected width on z=0 */
   GST_OBJECT_LOCK (canvas);
   GST_OBJECT_LOCK (viewport);
-  width = (canvas_width * viewport->projected_width) / canvas->width;
+
+  if (viewport->rotation == PGM_VIEWPORT_ROTATION_NONE
+      || viewport->rotation == PGM_VIEWPORT_ROTATION_180)
+    width = (canvas_width * viewport->projected_width) / canvas->width;
+  else
+    width = (canvas_width * viewport->projected_width) / canvas->height;
+
   GST_OBJECT_UNLOCK (viewport);
   GST_OBJECT_UNLOCK (canvas);
 
