@@ -1119,6 +1119,7 @@ static void
 pgm_gl_viewport_dispose (GObject *object)
 {
   PgmGlViewport *glviewport = PGM_GL_VIEWPORT (object);
+  PgmContextTask *task;
 
   GST_DEBUG_OBJECT (glviewport, "dispose");
 
@@ -1148,6 +1149,8 @@ pgm_gl_viewport_dispose (GObject *object)
   glviewport->update_queue = NULL;
   g_mutex_unlock (glviewport->update_lock);
 
+  task = pgm_context_task_new (PGM_CONTEXT_QUIT, NULL);
+  pgm_context_push_immediate_task (glviewport->context, task);
   pgm_context_free (glviewport->context);
 
   g_mutex_free (glviewport->layer_lock);
@@ -1214,6 +1217,8 @@ pgm_gl_viewport_class_init (PgmGlViewportClass *klass)
     GST_DEBUG_FUNCPTR (pgm_gl_drawable_set_size);
   klass->changed_func[PGM_DRAWABLE_POSITION] =
     GST_DEBUG_FUNCPTR (pgm_gl_drawable_set_position);
+  klass->changed_func[PGM_DRAWABLE_TRANSFORMATION_MATRIX] =
+    GST_DEBUG_FUNCPTR (pgm_gl_drawable_set_transformation_matrix);
   klass->changed_func[PGM_DRAWABLE_BG_COLOR] =
     GST_DEBUG_FUNCPTR (pgm_gl_drawable_set_bg_color);
   klass->changed_func[PGM_DRAWABLE_FG_COLOR] =
@@ -1232,8 +1237,12 @@ pgm_gl_viewport_class_init (PgmGlViewportClass *klass)
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_set_from_gst_buffer);
   klass->changed_func[PGM_IMAGE_DATA_PIXBUF] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_set_from_pixbuf);
+  klass->changed_func[PGM_IMAGE_DATA_SYSTEM_BUFFER] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_set_from_system_buffer);
   klass->changed_func[PGM_IMAGE_DATA_IMAGE] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_set_from_image);
+  klass->changed_func[PGM_IMAGE_SYSTEM_BUFFER_CONTENT] =
+    GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_system_buffer_changed);
   klass->changed_func[PGM_IMAGE_MAPPING_MATRIX] =
     GST_DEBUG_FUNCPTR ((PgmGlDrawableChangedFunc) pgm_gl_image_set_mapping_matrix);
   klass->changed_func[PGM_IMAGE_ALIGNMENT] =
@@ -1325,6 +1334,12 @@ pgm_gl_viewport_init (PgmGlViewport *glviewport)
   if (pgm_backend_has_alpha_component (glviewport->context->backend))
     glviewport->capacities |= PGM_VIEWPORT_OPACITY;
 
+#ifdef HAVE_GL_GLX_H
+  if (pgm_backend_has_system_buffer (glviewport->context->backend) &&
+      glviewport->context->feature_mask & PGM_GL_FEAT_TEXTURE_NON_POWER_OF_TWO)
+    glviewport->capacities |= PGM_VIEWPORT_X11_SYSTEM_BUFFER;
+#endif /* HAVE_GL_GLX_H */
+
   /* Drag status */
   glviewport->drag_status = FALSE;
 }
