@@ -120,22 +120,24 @@ typedef struct {
 
 /* GLX extensions checked at runtime */
 static ExtensionMap glx_extensions_map[] = {
-  { "GLX_SGIX_fbconfig",    PGM_GLX_FEAT_FBCONFIG     },
-  { "GLX_SGI_video_sync",   PGM_GLX_FEAT_VIDEO_SYNC   },
-  { "GLX_SGI_swap_control", PGM_GLX_FEAT_SWAP_CONTROL },
-  { NULL,                   0                         }
+  { "GLX_SGIX_fbconfig",           PGM_GLX_FEAT_FBCONFIG            },
+  { "GLX_SGI_video_sync",          PGM_GLX_FEAT_VIDEO_SYNC          },
+  { "GLX_SGI_swap_control",        PGM_GLX_FEAT_SWAP_CONTROL        },
+  { "GLX_EXT_texture_from_pixmap", PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP },
+  { NULL,                          0                                }
 };
 
 /* GLX functions binding */
 static PgmGlxBackendProcAddress glx_proc_address = {
   (PgmGlxChooseFbconfigFunc)        NULL,
-  (PgmGlxGetFbconfigsFunc)          NULL,
   (PgmGlxGetFbconfigAttribFunc)     NULL,
   (PgmGlxGetVisualFromFbconfigFunc) NULL,
   (PgmGlxCreateNewContextFunc)      NULL,
   (PgmGlxGetVideoSyncFunc)          NULL,
   (PgmGlxWaitVideoSyncFunc)         NULL,
-  (PgmGlxSwapIntervalFunc)          NULL
+  (PgmGlxSwapIntervalFunc)          NULL,
+  (PgmGlxBindTexImageFunc)          NULL,
+  (PgmGlxReleaseTexImageFunc)       NULL
 };
 
 /* X atom index IDs */
@@ -285,8 +287,6 @@ bind_glx_extensions (PgmGlxBackend *glxbackend)
         {
           glx->choose_fbconfig = (PgmGlxChooseFbconfigFunc)
             get_proc_address (glxbackend, "glXChooseFBConfig");
-          glx->get_fbconfigs = (PgmGlxGetFbconfigsFunc)
-            get_proc_address (glxbackend, "glXGetFBConfigs");
           glx->get_fbconfig_attrib = (PgmGlxGetFbconfigAttribFunc)
             get_proc_address (glxbackend, "glXGetFBConfigAttrib");
           glx->get_visual_from_fbconfig = (PgmGlxGetVisualFromFbconfigFunc)
@@ -298,8 +298,6 @@ bind_glx_extensions (PgmGlxBackend *glxbackend)
         {
           glx->choose_fbconfig = (PgmGlxChooseFbconfigFunc)
             get_proc_address (glxbackend, "glXChooseFBConfigSGIX");
-          glx->get_fbconfigs = (PgmGlxGetFbconfigsFunc)
-            get_proc_address (glxbackend, "glXGetFBConfigsSGIX");
           glx->get_fbconfig_attrib = (PgmGlxGetFbconfigAttribFunc)
             get_proc_address (glxbackend, "glXGetFBConfigAttribSGIX");
           glx->get_visual_from_fbconfig = (PgmGlxGetVisualFromFbconfigFunc)
@@ -308,9 +306,8 @@ bind_glx_extensions (PgmGlxBackend *glxbackend)
             get_proc_address (glxbackend, "glXCreateContextWithConfigSGIX");
         }
 
-      if ((!glx->choose_fbconfig) || (!glx->get_fbconfigs)
-          || (!glx->get_fbconfig_attrib) || (!glx->get_visual_from_fbconfig)
-          || (!glx->create_new_context))
+      if ((!glx->choose_fbconfig) || (!glx->get_fbconfig_attrib)
+          || (!glx->get_visual_from_fbconfig) || (!glx->create_new_context))
         {
           glxbackend->feature_mask &= ~PGM_GLX_FEAT_FBCONFIG;
         }
@@ -337,6 +334,18 @@ bind_glx_extensions (PgmGlxBackend *glxbackend)
       if (!glx->swap_interval)
         glxbackend->feature_mask &= ~PGM_GLX_FEAT_SWAP_CONTROL;
     }
+
+  /* Texture from pixmap */
+  if (glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP)
+    {
+      glx->bind_tex_image = (PgmGlxBindTexImageFunc)
+        get_proc_address (glxbackend, "glXBindTexImageEXT");
+      glx->release_tex_image = (PgmGlxReleaseTexImageFunc)
+        get_proc_address (glxbackend, "glXReleaseTexImageEXT");
+
+      if ((!glx->bind_tex_image) || (!glx->release_tex_image))
+        glxbackend->feature_mask &= ~PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP;
+    }
 }
 
 /* Get the GLX informations and load the extensions */
@@ -398,17 +407,28 @@ load_glx_extensions (PgmGlxBackend *glxbackend)
   return TRUE;
 }
 
-/* Retrieve the best visual supported by the OpenGL implementation that
- * matches our requirements. An ARGB visual could be searched on demand with
- * the "PGM_GL_ARGB_VISUAL" environment variable. This is sad but ARGB
- * visuals are not correctly supported by the DRI (the issue will be
- * fixed with redirected direct rendering and DRI2) and with the NVIDIA
- * proprietary driver there are vertical synchronization issues. */
-static XVisualInfo*
-get_visual_info (PgmGlxBackend *glxbackend)
+/* Retrieve the best framebuffer configuration supported by the OpenGL
+ * implementation that matches our requirements. The configuration is returned
+ * as a visual and FBConfig if supported. The visual and fbconfig returned must
+ * be freed after use. The returned boolean indicates whether the configuration
+ * retrieving succeed or not.
+ *
+ * An ARGB visual could be searched on demand with the "PGM_GL_ARGB_VISUAL"
+ * environment variable. This is sad but ARGB visuals are not correctly
+ *  supported by the DRI (the issue will be fixed with redirected direct
+ * rendering and DRI2) and with the NVIDIA proprietary driver there were last
+ * time I tried vertical synchronization issues.
+ */
+static gboolean
+get_framebuffer_configuration (PgmGlxBackend *glxbackend,
+                               XVisualInfo **visual,
+                               GLXFBConfig **fbconfig,
+                               gint *fbconfig_index)
 {
   PgmGlxBackendProcAddress *glx = glxbackend->glx;
-  XVisualInfo *visual = NULL;
+  XVisualInfo *_visual = NULL;
+  GLXFBConfig *_fbconfig = NULL;
+  gint _fbconfig_index = -1;
   gboolean use_argb_visual = FALSE;
   const gchar *argb_visual;
 
@@ -421,7 +441,6 @@ get_visual_info (PgmGlxBackend *glxbackend)
   if (glxbackend->feature_mask & PGM_GLX_FEAT_FBCONFIG)
     {
       gint render_event_base, render_error_base, nbconfigs;
-      GLXFBConfig *fbconfig = NULL;
 
       gint attrib[] = {
         GLX_DOUBLEBUFFER, True, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1,
@@ -432,48 +451,47 @@ get_visual_info (PgmGlxBackend *glxbackend)
       GST_DEBUG_OBJECT (glxbackend, "'GLX_SGIX_fbconfig' is available");
 
       /* Get the FBConfigs available on the system */
-      fbconfig = glx->choose_fbconfig (glxbackend->dpy, glxbackend->screen,
-                                       attrib, &nbconfigs);
-      if (!fbconfig)
-        {
-          GST_ERROR_OBJECT (glxbackend, "couldn't retrieve the fbconfigs");
-          goto error;
-        }
-      GST_DEBUG_OBJECT (glxbackend, "retrieved %d fbconfigs", nbconfigs);
+      _fbconfig = glx->choose_fbconfig (glxbackend->dpy, glxbackend->screen,
+                                        attrib, &nbconfigs);
+      if (!_fbconfig)
+        GST_WARNING_OBJECT (glxbackend, "couldn't retrieve the fbconfigs");
+      else
+        GST_DEBUG_OBJECT (glxbackend, "retrieved %d fbconfigs", nbconfigs);
 
       /* The user requested an ARGB visual and we have the RENDER extension,
        * sounds like good conditions to search for such a visual */
-      if (use_argb_visual && XRenderQueryExtension (glxbackend->dpy,
-                                                    &render_event_base,
-                                                    &render_error_base))
+      if (_fbconfig && use_argb_visual &&
+          XRenderQueryExtension (glxbackend->dpy,
+                                 &render_event_base,
+                                 &render_error_base))
         {
           XRenderPictFormat *format = NULL;
-          guint i;
+          gint i;
 
           /* Search for an ARGB visual */
           for (i = 0; i < nbconfigs; i++)
             {
-              visual = glx->get_visual_from_fbconfig (glxbackend->dpy,
-                                                      fbconfig[i]);
-              if (!visual)
+              _visual = glx->get_visual_from_fbconfig (glxbackend->dpy,
+                                                       _fbconfig[i]);
+              if (!_visual)
                 continue;
 
               format = XRenderFindVisualFormat (glxbackend->dpy,
-                                                visual->visual);
+                                                _visual->visual);
               if (!format)
                 {
-                  XFree (visual);
+                  XFree (_visual);
                   continue;
                 }
 
               if (format->direct.alphaMask > 0)
                 {
-                  GST_INFO_OBJECT (glxbackend, "found an ARGB visual", visual);
+                  GST_INFO_OBJECT (glxbackend, "found an ARGB visual", _visual);
                   glxbackend->feature_mask |= PGM_GLX_FEAT_ARGB_VISUAL;
                   break;
                 }
 
-              XFree (visual);
+              XFree (_visual);
             }
 
           /* If there's no ARGB visual, we use the first visual retrieved */
@@ -481,29 +499,39 @@ get_visual_info (PgmGlxBackend *glxbackend)
             {
               GST_DEBUG_OBJECT (glxbackend, "couldn't find an ARGB visual");
 
-              visual = glx->get_visual_from_fbconfig (glxbackend->dpy,
-                                                      fbconfig[0]);
-              if (visual)
+              i = 0;
+              _visual = glx->get_visual_from_fbconfig (glxbackend->dpy,
+                                                       _fbconfig[i]);
+              if (_visual)
                 GST_INFO_OBJECT (glxbackend, "found a RGB visual");
               else
-                GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
+                {
+                  GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
+                  return FALSE;
+                }
             }
+
+          _fbconfig_index = i;
         }
-      else
+      else if (_fbconfig)
         {
-          visual = glx->get_visual_from_fbconfig (glxbackend->dpy, fbconfig[0]);
-          if (visual)
+          _visual = glx->get_visual_from_fbconfig (glxbackend->dpy,
+                                                   _fbconfig[0]);
+          if (_visual)
             GST_INFO_OBJECT (glxbackend, "found a RGB visual");
           else
-            GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
-        }
+            {
+              GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
+              return FALSE;
+            }
 
-      XFree (fbconfig);
+          _fbconfig_index = 0;
+        }
     }
 
-  /* 'GLX_SGIX_fbconfig' not available, let the implementation choose a RGB
-   * visual for us */
-  else
+  /* 'GLX_SGIX_fbconfig' not available or the fbconfigs have not been correctly
+     retrieved, let the implementation choose a RGB visual for us */
+  if (!_fbconfig)
     {
       gint attrib[] = {
         GLX_DOUBLEBUFFER, 1, GLX_RED_SIZE, 1, GLX_GREEN_SIZE, 1,
@@ -511,15 +539,22 @@ get_visual_info (PgmGlxBackend *glxbackend)
 
       GST_DEBUG_OBJECT (glxbackend, "'GLX_SGIX_fbconfig' is not available");
 
-      visual = glXChooseVisual (glxbackend->dpy, glxbackend->screen, attrib);
-      if (visual)
+      _visual = glXChooseVisual (glxbackend->dpy, glxbackend->screen, attrib);
+      if (_visual)
         GST_INFO_OBJECT (glxbackend, "found a RGB visual");
       else
-        GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
+        {
+          GST_ERROR_OBJECT (glxbackend, "couldn't find any visual");
+          return FALSE;
+        }
     }
 
- error:
-  return visual;
+  /* Finally store the retrieved framebuffer configuration */
+  *visual = _visual;
+  *fbconfig = _fbconfig;
+  *fbconfig_index = _fbconfig_index;
+
+  return TRUE;
 }
 
 /* Setup the method used for the VBlank syncing */
@@ -584,16 +619,25 @@ setup_vblank (PgmGlxBackend *glxbackend)
       return;
     }
 
-  /* With the NVIDIA proprietary driver, with the GeForce 8000 GPUs, the
-   * video_sync extension works much better than for the previous one, that's
-   * good since the swap_control one takes all the CPU in fullscreen mode.
-   * Let's choose that extension for this class of GPUs. */
+  /* Workaround section for video cards/drivers dealing differently with the
+   * Vertical Blanking Interrupt:
+   *
+   * - With the NVIDIA proprietary driver, with the GeForce 8000 GPUs, the
+   *   video_sync extension works much better than for the previous one, that's
+   *   good since the swap_control one takes all the CPU in fullscreen mode.
+   *   Let's choose that extension for this class of GPUs.
+   *
+   * - With the Intel G33 series, the kernel freezes using swap_control
+   *   bad, bad, bad!!!
+   */
   if (glxbackend->feature_mask & PGM_GLX_FEAT_VIDEO_SYNC)
     {
       pgm_gl_get_string get_string = get_proc_address
         (glxbackend, "glGetString");
       const gchar *renderer = (const gchar*) get_string (GL_RENDERER);
-      if (!strncmp ("GeForce 8", renderer, 9))
+
+      if (!strncmp ("GeForce 8", renderer, 9)
+          || !strncmp ("Mesa DRI Intel(R) G33", renderer, 21))
         {
           GST_DEBUG_OBJECT (glxbackend, "Using the video_sync extension for "
                             "vblank syncing");
@@ -662,8 +706,6 @@ handle_wm_protocols_event (PgmGlxBackend *glxbackend,
       && xevent->xany.window == glxbackend->win)
     {
       pgmevent = pgm_event_new (PGM_DELETE);
-
-      ((PgmEventDelete *) pgmevent)->time = xevent->xclient.data.l[1];
       set_wm_user_time (glxbackend, xevent->xclient.data.l[1]);
     }
 
@@ -1221,6 +1263,7 @@ translate_motion_event (PgmGlxBackend *glxbackend,
   event->time = xevent->xmotion.time;
   event->x = xevent->xmotion.x;
   event->y = xevent->xmotion.y;
+  event->pressure = 0;
 }
 
 /* Fill a PgmEventButton giving the corresponding button xevent */
@@ -1232,6 +1275,7 @@ translate_button_press_event (PgmGlxBackend *glxbackend,
   event->time = xevent->xbutton.time;
   event->x = xevent->xbutton.x;
   event->y = xevent->xbutton.y;
+  event->pressure = 0;
 }
 
 /* Fill a PgmEventScroll giving the corresponding button xevent */
@@ -1289,6 +1333,7 @@ create_button_release_event (PgmGlxBackend *glxbackend,
   event->time = xevent->xbutton.time;
   event->x = xevent->xbutton.x;
   event->y = xevent->xbutton.y;
+  event->pressure = 0;
 
   return (PgmEvent *) event;
 }
@@ -1549,11 +1594,24 @@ event_prepare (GSource *source,
   PgmGlxBackendSource *glxbackend_source = (PgmGlxBackendSource *) source;
   PgmGlxBackend *glxbackend = glxbackend_source->glxbackend;
   PgmBackend *backend = PGM_BACKEND (glxbackend);
+  gint requested_fps = backend->context->requested_fps;
 
   if (XEventsQueued (glxbackend->dpy, QueuedAlready))
     return TRUE;
 
-  *timeout = backend->context->update_tag ? 0 : -1;
+  /* If there are rendering updates */
+  if (backend->context->update_tag)
+    {
+      /* If the rendering is blocked on the vblank, we give a timeout of 0 */
+      if (requested_fps == 0)
+        *timeout = 0;
+      /* If the rendering is done in a timeout, we use the same timeout */
+      else
+        *timeout = requested_fps;
+    }
+  /* If there are no rendering updates, we poll with no timeout */
+  else
+    *timeout = -1;
 
   return FALSE;
 }
@@ -1604,6 +1662,7 @@ add_event_source (PgmGlxBackend *glxbackend)
   /* Attach it */
   g_source_set_callback ((GSource *) source, (GSourceFunc) dispatch_x_events,
                          glxbackend, NULL);
+  g_source_set_priority ((GSource *) source, G_PRIORITY_DEFAULT-20);
   glxbackend->event_id = g_source_attach ((GSource *) source, render_context);
   g_source_unref ((GSource *) source);
 
@@ -1659,7 +1718,6 @@ static gboolean
 pgm_glx_backend_create_window (PgmBackend *backend)
 {
   PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
-  XVisualInfo *visual = NULL;
   guchar xdnd_version = XDND_PROTOCOL_VERSION;
   const gchar *indirect;
   XSetWindowAttributes attr;
@@ -1686,11 +1744,12 @@ pgm_glx_backend_create_window (PgmBackend *backend)
       return FALSE;
     }
 
-  /* Visual retrieving */
-  visual = get_visual_info (glxbackend);
-  if (!visual)
+  /* Framebuffer configuration retrieving */
+  if (!get_framebuffer_configuration (glxbackend, &glxbackend->visual,
+                                      &glxbackend->fbconfig,
+                                      &glxbackend->fbconfig_index))
     {
-      GST_ERROR_OBJECT (glxbackend, "couldn't get visual");
+      GST_ERROR_OBJECT (glxbackend, "couldn't get framebuffer configuration");
       return FALSE;
     }
 
@@ -1698,10 +1757,11 @@ pgm_glx_backend_create_window (PgmBackend *backend)
   pgm_viewport_get_size (viewport, &width, &height);
   memset (&attr, 0, sizeof (XSetWindowAttributes));
   attr.colormap = XCreateColormap (glxbackend->dpy, glxbackend->root,
-                                   visual->visual, AllocNone);
+                                   glxbackend->visual->visual, AllocNone);
   glxbackend->win = XCreateWindow (glxbackend->dpy, glxbackend->root,
-                                   0, 0, width, height, 0, visual->depth,
-                                   InputOutput, visual->visual,
+                                   0, 0, width, height, 0,
+                                   glxbackend->visual->depth, InputOutput,
+                                   glxbackend->visual->visual,
                                    CWBackPixel | CWBorderPixel | CWColormap,
                                    &attr);
   if (!glxbackend->win)
@@ -1733,18 +1793,17 @@ pgm_glx_backend_create_window (PgmBackend *backend)
   /* Create the OpenGL context */
   indirect = g_getenv ("PGM_GL_INDIRECT_RENDERING");
   if (indirect && *indirect == '1')
-    glxbackend->ctx = glXCreateContext (glxbackend->dpy, visual, NULL,
-                                        PGM_GL_FALSE);
+    glxbackend->ctx = glXCreateContext (glxbackend->dpy, glxbackend->visual,
+                                        NULL, PGM_GL_FALSE);
   else
-    glxbackend->ctx = glXCreateContext (glxbackend->dpy, visual, NULL,
-                                        PGM_GL_TRUE);
+    glxbackend->ctx = glXCreateContext (glxbackend->dpy, glxbackend->visual,
+                                        NULL, PGM_GL_TRUE);
   if (!glxbackend->ctx)
     {
       GST_ERROR_OBJECT (glxbackend, "couldn't create OpenGL context");
       return FALSE;
     }
   glXMakeCurrent (glxbackend->dpy, glxbackend->win, glxbackend->ctx);
-  XFree (visual);
 
   /* Is it a direct rendering context? */
   if (glXIsDirect (glxbackend->dpy, glxbackend->ctx))
@@ -1793,7 +1852,9 @@ pgm_glx_backend_destroy_window (PgmBackend *backend)
 
       /* Clean up X related data */
       XFreeCursor (glxbackend->dpy, glxbackend->none_cursor);
+      XFree (glxbackend->fbconfig);
       glXDestroyContext (glxbackend->dpy, glxbackend->ctx);
+      XFree (glxbackend->visual);
       XDestroyWindow (glxbackend->dpy, glxbackend->win);
       XCloseDisplay (glxbackend->dpy);
       glxbackend->created = FALSE;
@@ -2296,6 +2357,140 @@ pgm_glx_backend_has_alpha_component (PgmBackend *backend)
   return glxbackend->feature_mask & PGM_GLX_FEAT_ARGB_VISUAL;
 }
 
+static gboolean
+pgm_glx_backend_has_system_buffer (PgmBackend *backend)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  GST_LOG_OBJECT (backend, "has_system_buffer");
+
+  return glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP;
+}
+
+gpointer
+pgm_glx_backend_create_system_buffer_object (PgmBackend *backend,
+                                             gconstpointer system_buffer,
+                                             PgmImagePixelFormat format)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+  GLXPixmap *glxpixmap = NULL;
+
+  /* Cards with no support for non-power-of-two textures are not supported,
+   * the plugin already checked that for us */
+  int attrib[] = {
+    GLX_TEXTURE_TARGET_EXT, GLX_TEXTURE_2D_EXT,
+    GLX_TEXTURE_FORMAT_EXT, 0,
+    GLX_MIPMAP_TEXTURE_EXT, 0,
+    None
+  };
+
+  GST_LOG_OBJECT (glxbackend, "create_system_buffer_object");
+
+  if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
+    {
+      GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
+                          "the GLX_EXT_texture_from_pixmap extension");
+      goto error;
+    }
+
+  if (G_UNLIKELY (!glxbackend->fbconfig ||
+                  !glxbackend->fbconfig[glxbackend->fbconfig_index]))
+    {
+      GST_WARNING_OBJECT (glxbackend, "cannot create a system buffer without "
+                          "valid fbconfig");
+      goto error;
+    }
+
+  /* Setup the correct RGB or RGBA format */
+  if (format == PGM_IMAGE_RGB)
+    attrib[3] = GLX_TEXTURE_FORMAT_RGB_EXT;
+  else if (format == PGM_IMAGE_RGBA)
+    attrib[3] = GLX_TEXTURE_FORMAT_RGBA_EXT;
+  else
+    {
+      GST_WARNING_OBJECT (glxbackend, "color space other than RGB and RGBA are "
+                          "not accepted for system buffer");
+      goto error;
+    }
+
+  glxpixmap = g_slice_new (GLXPixmap);
+
+  *glxpixmap = glXCreatePixmap (glxbackend->dpy,
+                                glxbackend->fbconfig[glxbackend->fbconfig_index],
+                                *((Pixmap*) system_buffer), attrib);
+  XSync (glxbackend->dpy, False);
+
+ error:
+  return glxpixmap;
+}
+
+void
+pgm_glx_backend_destroy_system_buffer_object (PgmBackend *backend,
+                                              gpointer system_buffer_object)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  g_return_if_fail (system_buffer_object);
+
+  GST_LOG_OBJECT (glxbackend, "destroy_system_buffer_object");
+
+  if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
+    {
+      GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
+                          "the GLX_EXT_texture_from_pixmap extension");
+      return;
+    }
+
+  glXDestroyPixmap (glxbackend->dpy, *((GLXPixmap*) system_buffer_object));
+
+  g_slice_free (GLXPixmap, system_buffer_object);
+  system_buffer_object = NULL;
+}
+
+void
+pgm_glx_backend_bind_system_buffer_object (PgmBackend *backend,
+                                           gconstpointer system_buffer_object)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  g_return_if_fail (system_buffer_object);
+
+  GST_LOG_OBJECT (glxbackend, "bind_system_buffer_object");
+
+  if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
+    {
+      GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
+                          "the GLX_EXT_texture_from_pixmap extension");
+      return;
+    }
+
+  glxbackend->glx->bind_tex_image (glxbackend->dpy,
+                                   *((GLXPixmap*) system_buffer_object),
+                                   GLX_FRONT_LEFT_EXT, NULL);
+}
+
+void
+pgm_glx_backend_release_system_buffer_object (PgmBackend *backend,
+                                              gconstpointer system_buffer_object)
+{
+  PgmGlxBackend *glxbackend = PGM_GLX_BACKEND (backend);
+
+  g_return_if_fail (system_buffer_object);
+
+  GST_LOG_OBJECT (glxbackend, "release_system_buffer_object");
+
+  if (!(glxbackend->feature_mask & PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP))
+    {
+      GST_WARNING_OBJECT (glxbackend, "the GLX implementation does not support "
+                          "the GLX_EXT_texture_from_pixmap extension");
+      return;
+    }
+
+  glxbackend->glx->release_tex_image (glxbackend->dpy,
+                                      *((GLXPixmap*) system_buffer_object),
+                                      GLX_FRONT_LEFT_EXT);
+}
+
 /* GObject stuff */
 
 PGM_DEFINE_DYNAMIC_TYPE (PgmGlxBackend, pgm_glx_backend, PGM_TYPE_BACKEND);
@@ -2391,6 +2586,16 @@ pgm_glx_backend_class_init (PgmGlxBackendClass *klass)
     GST_DEBUG_FUNCPTR (pgm_glx_backend_get_embedding_id);
   backend_class->has_alpha_component =
     GST_DEBUG_FUNCPTR (pgm_glx_backend_has_alpha_component);
+  backend_class->has_system_buffer =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_has_system_buffer);
+  backend_class->create_system_buffer_object =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_create_system_buffer_object);
+  backend_class->destroy_system_buffer_object =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_destroy_system_buffer_object);
+  backend_class->bind_system_buffer_object =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_bind_system_buffer_object);
+  backend_class->release_system_buffer_object =
+    GST_DEBUG_FUNCPTR (pgm_glx_backend_release_system_buffer_object);
 }
 
 static void
@@ -2406,6 +2611,9 @@ pgm_glx_backend_init (PgmGlxBackend *glxbackend)
 
   glxbackend->dpy = NULL;
   glxbackend->glx = NULL;
+  glxbackend->visual = NULL;
+  glxbackend->fbconfig = NULL;
+  glxbackend->fbconfig_index = -1;
   glxbackend->atom = NULL;
   glxbackend->startup_notification_id = NULL;
   glxbackend->embedder = None;
