@@ -56,11 +56,12 @@ typedef struct _PgmGlxBackendProcAddress PgmGlxBackendProcAddress;
 
 /* Feature flags */
 typedef enum {
-  PGM_GLX_FEAT_FBCONFIG         = (1 << 0),
-  PGM_GLX_FEAT_VIDEO_SYNC       = (1 << 1),
-  PGM_GLX_FEAT_SWAP_CONTROL     = (1 << 2),
-  PGM_GLX_FEAT_DIRECT_RENDERING = (1 << 3),
-  PGM_GLX_FEAT_ARGB_VISUAL      = (1 << 4)
+  PGM_GLX_FEAT_FBCONFIG            = (1 << 0),
+  PGM_GLX_FEAT_VIDEO_SYNC          = (1 << 1),
+  PGM_GLX_FEAT_SWAP_CONTROL        = (1 << 2),
+  PGM_GLX_FEAT_DIRECT_RENDERING    = (1 << 3),
+  PGM_GLX_FEAT_ARGB_VISUAL         = (1 << 4),
+  PGM_GLX_FEAT_TEXTURE_FROM_PIXMAP = (1 << 5)
 } PgmGlxBackendFeature;
 
 /* VBlank modes */
@@ -78,7 +79,6 @@ struct _PgmGlxBackend {
   Window   win;
   gint     screen;
   gint     root;
-  XSetWindowAttributes attr;
 
   /* X protocols */
   Atom   *atom;
@@ -94,6 +94,9 @@ struct _PgmGlxBackend {
 
   /* OpenGL context handling */
   PgmGlxBackendProcAddress *glx;
+  GLXFBConfig              *fbconfig;
+  gint                      fbconfig_index;
+  XVisualInfo              *visual;
   GLXContext                ctx;
 
   /* GLX informations */
@@ -148,9 +151,6 @@ typedef GLXFBConfig* (*PgmGlxChooseFbconfigFunc)        (Display *dpy,
                                                          gint screen,
                                                          const gint *attrib_list,
                                                          gint *nelements);
-typedef GLXFBConfig* (*PgmGlxGetFbconfigsFunc)          (Display *dpy,
-                                                         gint screen,
-                                                         gint *nelements);
 typedef gint         (*PgmGlxGetFbconfigAttribFunc)     (Display *dpy,
                                                          GLXFBConfig config,
                                                          gint attribute,
@@ -167,17 +167,26 @@ typedef gint         (*PgmGlxWaitVideoSyncFunc)         (gint divisor,
                                                          gint remainder,
                                                          guint *count);
 typedef gint         (*PgmGlxSwapIntervalFunc)          (gint interval);
+typedef void         (*PgmGlxBindTexImageFunc)          (Display *dpy,
+                                                         GLXDrawable drawable,
+                                                         int buffer,
+                                                         const int *attrib_list);
+typedef void         (*PgmGlxReleaseTexImageFunc)       (Display *dpy,
+                                                         GLXDrawable drawable,
+                                                         int buffer);
+
 
 /* GLX function pointers used for extensions loading */
 struct _PgmGlxBackendProcAddress {
   PgmGlxChooseFbconfigFunc        choose_fbconfig;
-  PgmGlxGetFbconfigsFunc          get_fbconfigs;
   PgmGlxGetFbconfigAttribFunc     get_fbconfig_attrib;
   PgmGlxGetVisualFromFbconfigFunc get_visual_from_fbconfig;
   PgmGlxCreateNewContextFunc      create_new_context;
   PgmGlxGetVideoSyncFunc          get_video_sync;
   PgmGlxWaitVideoSyncFunc         wait_video_sync;
   PgmGlxSwapIntervalFunc          swap_interval;
+  PgmGlxBindTexImageFunc          bind_tex_image;
+  PgmGlxReleaseTexImageFunc       release_tex_image;
 };
 
 GType       pgm_glx_backend_get_type (void) G_GNUC_CONST;
