@@ -57,6 +57,7 @@ typedef enum {
   PGM_TEXTURE_BUFFER,
   PGM_TEXTURE_GST_BUFFER,
   PGM_TEXTURE_PIXBUF,
+  PGM_TEXTURE_SYSTEM_BUFFER
 } PgmTextureStorageType;
 
 struct _PgmTexture {
@@ -66,6 +67,7 @@ struct _PgmTexture {
     gpointer   buffer;
     GstBuffer *gstbuffer;
     GdkPixbuf *pixbuf;
+    gpointer   systembuffer;
   } data;
 
   PgmImagePixelFormat csp; /* Color space */
@@ -75,8 +77,8 @@ struct _PgmTexture {
   gint   height_pot;       /* Power-of-two height of the buffer */
   gfloat norm_width;       /* width / width_pot */
   gfloat norm_height;      /* height / height_pot */
-  gfloat inv_norm_width;   /* (width / width_pot)^1 */
-  gfloat inv_norm_height;  /* (height / height_pot)^1 */
+  gfloat inv_norm_width;   /* (width / width_pot)^-1 */
+  gfloat inv_norm_height;  /* (height / height_pot)^-1 */
   gint   stride;           /* Stride of the buffer (not used) */
   gint   size;             /* Size of the buffer in bytes */
 
@@ -115,6 +117,13 @@ void        pgm_texture_set_buffer        (PgmTexture *texture,
                                            guint size,
                                            guint stride);
 
+/* Set a system buffer */
+void        pgm_texture_set_system_buffer (PgmTexture *texture,
+                                           gconstpointer system_buffer,
+                                           PgmImagePixelFormat csp,
+                                           guint width,
+                                           guint height);
+
 /* Set a GdkPixbuf */
 void        pgm_texture_set_pixbuf        (PgmTexture *texture,
                                            GdkPixbuf *pixbuf);
