@@ -65,14 +65,10 @@ struct _PgmWglBackend {
   HGLRC                 hrc;
   HDC                   hdc;
   PIXELFORMATDESCRIPTOR pfd;
-  RECT                  rect;
-  DWORD                 dwstyle;
   guint                 pf;
 
-  /* WGL informations */
-  const gchar *vendor;
+  /* WGL extensions */
   const gchar *extensions;
-  gfloat       version;
 
   /* Message source id */
   guint message_id;
@@ -103,8 +99,11 @@ struct _PgmWglBackend {
 
   /* States */
   gboolean created;
+  gboolean never_shown;
   gboolean hidden;
   gboolean fullscreen;
+  gboolean decorated;
+  gboolean intel_chipset;
 };
 
 struct _PgmWglBackendClass {
